/**
 * @preamble
 * @description This object provides a façade to the objects & methods that make up the core of the player,
 * allowing content pages or custom theme logic to interact with the player & it's data.
 * @file /player/scripts/contentApi.js
 */
var contentApi = (function () {
	var player;
	var behaviour;
	var navigation;
	var dataStorage;
	var scormAPI;
	var data;
	var returnObj = {DkiApi: {}};
	var eventHandlers = {};
	var pub = {
		/**
		 * The initializer, takes in the player object and, if applicable, the scormAPI object
		 * @method init 
		 * @param playerObj [object] The object providing core player functionality.
		 * @param scormAPIObj [object] [optional] The object providing the SCORM API methods, if this is a 
		 * SCORM course
		 */
		init: function (playerObj, scormAPIObj) {
			player = playerObj;
			strings = playerObj.strings;
			navigation = player.navigation;
			dataStorage = player.dataStorage;
			scormAPI = scormAPIObj;
			behaviour = player.behaviour;
			data = {};
		},
		/**
		 * Enables the course player's audio button
		 * @method audioButtonOn
		 */
		audioButtonOn: function () {
			navigation.showAudioButton();
		},
		/**
		 * Disables the course player's audio button
		 * @method audioButtonOff
		 */
		audioButtonOff: function () {
			navigation.hideAudioButton();
		},
		/**
		 * Navigates to the previous content page
		 * @method contentGoBack
		 */
		contentGoBack: function () {
			if (!player.inAssessment && !player.inReview) {
				navigation.fireBack();
			}
		},
		/**
		 * Navigates to the next content page
		 * @method contentGoNext
		 */
		contentGoNext: function () {
			if (!player.inAssessment && !player.inReview) {
				navigation.fireForward();
			}
		},
		/**
		 * Returns the audio status
		 * @method getAudioStatus
		 * @return boolean The current status of the audio, false for muted or true for on.
		 */
		getAudioStatus: function () {
			return player.audio;
		},
		/**
		 * Returns the percentage of the course currently complete (0 - 100)
		 * @method getCourseCompletion
		 * @return integer The percentage of the course content currently complete.
		 */
		getCourseCompletion: function () {
			return dataStorage.getCourseCompletion();
		},
		/**
		 * Returns the course's name
		 * @method getCourseName
		 * @return string The name of the course
		 */
		getCourseName: function () {
			return dataStorage.courseStructure.name;
		},
		/**
		 * Returns the number of content pages in the course
		 * @method getCoursePageCount
		 * @return integer The number of content pages in the course.
		 */
		getCoursePageCount: function () {
			return dataStorage.countCoursePages();
		},
		/**
		 * Returns the current course score.  This is the actual score, out of 100
		 * @method getCourseScore
		 * @return integer The current course score, out of 100
		 */
		getCourseScore: function () {
			return dataStorage.getCourseScoreAvg();
		},
		/**
		 * Returns the current average score.  This is only calculated for currently attempted tests
		 * @method getCourseScoreAvg
		 * @return integer The average score for all currently attempted tests
		 */
		getCourseScoreAvg: function () {
			return dataStorage.getCourseScoreAvg();
		},
		/**
		 * Returns the name of the current module.
		 */
		getCurrentModuleName: function () {
			return dataStorage.currentModule.name;
		},
		/**
		 * Retrieves a data value
		 * @method getData
		 * @param label [string] The label of the data value to return
		 */
		getData: function (label) {
			return data[label];
		},
		/**
		 * Returns the duration of the course, in minutes
		 * @method getDuration
		 * @return integer The number of minutes the course is expected to take to complete
		 */
		getDuration: function () {
			return parseInt(dataStorage.courseStructure.duration, 10);
		},
		/**
		 * Returns the maximum number of allowed post-test attempts for the course
		 * @return integer
		 */
		getMaxTestAttempts: function () {
			if (!behaviour.enablePostTest) {
				return 0;
			}
			else if (!behaviour.allowRetesting) {
				return 1;
			}
			else {
			return parseInt(behaviour.retestAttempts, 10) + 1;
			}
		},
		/**
		 * Returns the course modules, in order
		 * @return object[] Module objects.  Each object has id, name, hasTesting, pre {score, rawScore, maxScore} and 
		 * post {attempts, score, rawScore, maxScore}.  Pre and post only exist if anything has been recorded.
		 */
		getModules: function () {
			var modules = [];
			var currentMod;
			for (var i = 0; i < dataStorage.courseStructure.modules.length; i += 1) {
				currentMod = dataStorage.courseStructure.modules[i];
				modules[i] = {
					name : currentMod.name,
					id: currentMod.id,
					hasTesting: currentMod.weight === 1 ? true : false,
					remainingAttempts: dataStorage.remainingPostAttempts(currentMod)
				};
				if (currentMod.pre) {
					modules[i].pre = {
						score: currentMod.pre.score,
						rawScore: currentMod.pre.points,
						maxScore: currentMod.pre.weight
					};
				}
				if (currentMod.post) {
					modules[i].post = {
						score: currentMod.post.score,
						attempts: currentMod.post.attempts,
						rawScore: currentMod.post.points,
						maxScore: currentMod.post.weight
					};
				}
			}

			return modules;
		},
		/**
		 * Returns the score for a course module
		 * @method getModuleScore
		 * @param moduleID [integer] [optional] The ID of the module.  If omitted, the score for the current
		 * module is returned.
		 * @return integer The score for the module
		 */
		getModuleScore: function (moduleID) {
			if (moduleID === undefined || moduleID === null) {
				moduleID = dataStorage.currentModule.id;
			}
			return dataStorage.getModuleScore(moduleID);
		},
		/**
		 * Returns the number of modules in the course with attempted tests
		 * @method getModuleTestCompleteCount
		 * @return integer The number of modules in the course with attempted tests
		 */
		getModuleTestCompleteCount: function () {
			return dataStorage.modulesTestAttempted().length;
		},
		/**
		 * Returns the number of modules in the course that have testing
		 * @method getModuleTestCount
		 * @return integer The number of modules in the course with testing
		 */
		getModuleTestCount: function () {
			return dataStorage.modulesWithTesting().length;
		},
		/**
		 * Returns the score needed to record a pass in the course
		 * @method getPassMark
		 * @return integer The score needed to record a pass in the course
		 */
		getPassMark: function () {
			return dataStorage.passMark;
		},
		/**
		 * Returns the current status for the course.  Will pull from LMS for a SCORM course
		 * @method getStatus
		 * @return string One of 'passed'/'failed'/'complete'/'incomplete'
		 */
		getStatus: function () {
			if (scormAPI) {
				switch(scormAPI.GetStatus()) {
					case scormAPI.LESSON_STATUS_PASSED:
						return "passed";
					case scormAPI.LESSON_STATUS_COMPLETED:
						return "complete";
					case scormAPI.LESSON_STATUS_FAILED:
						return "failed";
					default:
						return "incomplete";
				}
			}
			else {
				return dataStorage.lessonStatus;
			}
		},
		/**
		 * Gets the name of the current student.  Will pull from LMS for a SCORM course, otherwise will prompt.
		 * This method is asynchronous with the possiblity of using a non-native prompt, therefor doesn't directly 
		 * a value
		 * @method getStudentName
		 * @param callback [function] The function that will handle the name value.  It will accept a single argument,
		 * the name of the student.
		 */
		getStudentName: function (callback) {
			var name;
			var promptCallback = function (result, input) {
				if (result) {
					name = input;
				}
				else {
					name = "";
				}
				callback(name);
			};
			if (scormAPI) {
				callback(scormAPI.GetStudentName());
			}
			else {
				DKI.prompt(strings.txtEnterName, promptCallback);
			}
		},
		/**
		 * Hides the course player's 'back' button
		 * @method hideBack
		 */
		hideBack: function () {
			navigation.hideBackButton();
		},
		/**
		 * Hides both navigation buttons, 'back' and 'forward'
		 * @method hideBoth
		 */
		hideBoth: function () {
			navigation.hideBackButton();
			navigation.hideForwardButton();
		},
		/**
		 * Hides the course player's 'forward' button
		 * @method hideForward
		 */
		hideForward: function () {
			navigation.hideForwardButton();
		},
		/**
		 * Hides the transcript tab if currently shown
		 * @method hideTranscript
		 */
		hideTranscript: function () {
			player.hideTranscript();
		},
		/**
		 * Jumps to a given page
		 * @method jumptoSubeo
		 * @param subeoID [integer] The ID of the page to jump to
		 */
		jumpToSubeo: function(subeoId){
			if (!player.inAssessment && !player.inReview) {
				player.jumpToSubeo(subeoId);
			}
		},
		playerEvent: {
			/**
			 * Subscribes to a named player event
			 * @method playerEvent.on
			 * @param name [string] The name of the event to subscribe to
			 * @param handler [function] The event handler function
			 */
			on: function (name, handler) {
				$(player).bind(name, handler);
			},
			/**
			 * Removes handler from a named player event
			 * @method playerEvent.remove
			 * @param name [string] The name of the event to remove handler from
			 * @param handler [function] [optional] If provided, this specific handler will be removed.  Otherwise, 
			 * all handlers will be removed
			 */
			remove: function (name, handler) {
				$(player).unbind(name, handler);
			}
		},
		/**
		 * Prints the current content page
		 * @method printPage
		 */
		printPage: function () {
			window.print();
		},
		/**
		 * Saves an arbitrary data value
		 * @method setData
		 * @param label [string] The used to save the value
		 * @param value [object] The value to save
		 */
		setData: function (label, value) {
			data[label] = value;
		},
		/**
		 * Sets the current learning object's status to complete
		 * @param setObjectCompleted
		 */
		setObjectCompleted: function () {
			player.setCurrentObjectComplete();
		},
		/**
		 * Displays the course player's back button
		 * @method showBack
		 */
		showBack: function () {
			player.navigation.showBackButton();
		},
		/**
		 * Displays the course player's navigation buttons
		 * @method showBoth
		 */
		showBoth: function () {
			player.navigation.showBackButton();
			player.navigation.showForwardButton();
		},
		/**
		 * Displays the course player's forward button
		 * @method showForward
		 */
		showForward: function () {
			player.navigation.showForwardButton();
		},
		/**
		 * If the current module has testing, mark all learning complete and launch the test
		 * @method takeTest
		*/
		takeTest: function () {
			if (player.inAssessment || player.inReview) {
				return;
			}
			var currentModule = dataStorage.currentModule;
			var moduleHasTest = dataStorage.moduleHasTest(currentModule);
			var testingEnabled =behaviour.enablePostTest;
			if (!testingEnabled) {
				alert("Testing is not enabled for this course.");
				return null;
			}
			else if (!moduleHasTest) {
				alert("This module does not have any testing.");
				return null;
			}
			else if (dataStorage.isMaxedOut(currentModule)) {
				alert("You have exceeded the maximum number of test attempts for this module.");
				return null;
			}
			dataStorage.markModuleComplete(currentModule);
			dataStorage.jumpToPage(dataStorage.getModuleEnd(currentModule));
			player.launchTest(currentModule.id, "post");
			return null
		},
		/**
		 * Toggles the visible state of the transcript tab
		 * @method toggleTranscript
		 */
		toggleTranscript: function () {
			player.toggleTranscript();
		},
		/**
		 * Shows the transcript tab
		 * @method showTranscript
		 */
		showTranscript: function () {
			player.showTranscript();
		},
		/**
		 * Upens a url on a modal
		 * @method openModal
		 * @param url [string] The url to open
		 */
		openModal: function(url){
			player.openModal(url);
		}
	};
	for (var method in pub) {
		//We want to expost all the public methods on a DkiApi key for legacy reasons, but also 
		//on the root of the contentApi object, because that makes more sense to work with
		if (pub.hasOwnProperty(method)) {
			returnObj[method] = pub[method];
			returnObj.DkiApi[method] = pub[method];
		}
	}
	returnObj.shLMS = {
		meth1: function (attempts, id) {
			var dataStorage = player.dataStorage;
			var currentModule = id ? dataStorage.findModule(id) : dataStorage.currentModule;
			if (currentModule.post) {
				currentModule.post.attempts = attempts;
			}

			if (dataStorage.saveCompletion) {
				dataStorage.saveCompletion();
			}
		}
	};
	return returnObj;
})();
