var DKI;
if (!DKI) {
	DKI = {};
}

/*
 * Provides an object for tracking/navigating through the structure of a course
 * @constructor
 */
DKI.DataStorage = $.Class({
	init: function (courseStructure, behaviour, aStrings, startPageId) {
		this.courseStructure = this.initStructure(courseStructure);
		this.behaviour = behaviour;
		this.strings = aStrings;
		this.currentModuleIndex = 0;
		this.currentObjectIndex = 0;
		this.currentPageIndex = 0;
		//if we pass in an ID for the page to start at, set the player to start from there.
		//note this does NOT include questions. 
		if(startPageId){
			this.bookmark = startPageId;
		}
		this.currentModule = this.courseStructure.modules[this.currentModuleIndex];
		this.currentObject = this.currentModule.objects[this.currentObjectIndex];
		this.currentPage = this.currentObject.pages[this.currentPageIndex];
		this.moduleCount = this.courseStructure.modules.length;
		this.currentObjectCount = this.currentModule.objects.length;
		this.currentPageCount = this.currentObject.pages.length;
		this.previousPage = null;
		//if we're not on the first page, get the previous one
		if(this.currentPageIndex > 0){
			this.previousPage = this.getPreviousPage();
		}		
		this.nextPage = this.getNextPage();
		this.currentModulePageTotal = this.countModulePages(this.currentModule);
		this.currentModulePage = this.currentPageIndex + 1;
		this.passMark = this.courseStructure.passMark;
		this.assessment;
		this.lessonStatus = "incomplete";
	}
});

/**
 * Returns true if all course tests have been attempted (or course has not testing), false 
 * otherwise
 *
 * @return Boolean
 */
DKI.DataStorage.prototype.allTestsAttempted = function () {
	var totalTests;
	var attemptedTests;
	if (!this.hasTesting() || !(this.behaviour.enablePreTest || this.behaviour.enablePostTest)) {
		return true;
	}
	else {
		totalTests = this.modulesWithTesting().length;
		attemptedTests = this.modulesTestAttempted().length;

		return attemptedTests == totalTests;
	}
};

/**
 * Factory method, creates an Assessment object for a given module
 * @param module Object The module the assessment is being created for
 * @param prePost String One of "pre" or "post", for the assessment type
 * @param test Object The test object
 * @return DKI.Assessment
 */
DKI.DataStorage.prototype.createAssessment = function (module, prePost, test) {
	return new DKI.Assessment(module, prePost, test);
};

/**
 * For a given object, checks child pages.  If all pages are complete, the object is marked as complete
 * @param object Object The learning object whose complete is checked.
 */
DKI.DataStorage.prototype.checkObjectComplete = function (object) {
	var pageCount = object.pages.length;
	var total = 0;
	for (var i = 0; i < pageCount; i ++) {
		if (object.pages[i].complete) {
			total += 1;
		}
	}

	if (total === pageCount) {
		this.markObjectComplete(object);
	}
};

/**
 * For a given module, checks child objects.  If all children are complete, the module is marked as complete
 * @param module Object The module to check for completion
 */
DKI.DataStorage.prototype.checkModuleComplete = function (module) {
	var objCount = module.objects.length;
	var total = 0;
	for (var i = 0; i < objCount; i ++) {

		if (module.objects[i].complete) {
			total += 1;
		}
	}

	if (total === objCount) {
		this.markModuleComplete(module);
	}
};

/**
 * Checks all modules.  If all are complete, then fires the course complete event
 * @fires courseCompleted
 */
DKI.DataStorage.prototype.checkCourseComplete = function () {
	var modCount = this.courseStructure.modules.length;
	var total = 0;
	for (var i = 0; i < modCount; i ++) {
		if (this.courseStructure.modules[i].complete) {
			total += 1;
		}
	}

	if (total === modCount) {
		$(this).trigger(DKI.DataStorage.events.courseCompleted);
		this.doSetCourseComplete();
	}
};

/**
 * Returns a count of the total number of pages in the course
 * @returns Integer The number of pages in the course.
 */
DKI.DataStorage.prototype.countCoursePages = function () {
	var pageCount = 0;
	for (var i = 0; i < this.courseStructure.modules.length; i += 1) {
		pageCount += this.countModulePages(this.courseStructure.modules[i]);
	}

	return pageCount;
};

/**
 * Returns the number of pages in a module
 * @param module Object
 * @returns Integer The number of 
 */
DKI.DataStorage.prototype.countModulePages = function (module) {
	var pageCount = 0;
	for (var i = 0; i < module.objects.length; i++) {
		pageCount += module.objects[i].pages.length;
	}

	return pageCount;
};


/**
 * Creates a preTest attempt for a module
 * @param moduleID String
 * @calls createTest
 */
DKI.DataStorage.prototype.createPreTest = function (moduleID) {
	this.createTest(moduleID, "pre");
};

/**
 * Creates a postTest attempt for a module
 * @param moduleID String
 * @calls createTest
 */
DKI.DataStorage.prototype.createPostTest = function (moduleID) {
	this.createTest(moduleID, "post");
};

/**
 * Creates a test attempt for a module
 * @param moduleID String
 * @param prePost String one of "pre" or "post"
 */
DKI.DataStorage.prototype.createTest = function (moduleID, prePost) {
	var module = typeof moduleID === "object" ? moduleID : this.findModule(moduleID);
	var test = this.loadTest(module);
	var canTest = true;
	
	if (prePost === "pre") {
		if (this.remainingPreAttempts(module) > 0) {
			module.pre = {score: 0};
		}
		else {
			canTest = false;
		}
	}
	else {
		if (this.remainingPostAttempts(module) > 0) {
			if (module.post) {
				module.post.attempts++;
			}
			else {
				module.post = {
					attempts: 1,
					score: 0
				}
			}
		}
		else {
			canTest = false;
		}
	}

	if (canTest) {
		this.assessment = this.createAssessment(module, prePost, test);
		$(this).trigger(DKI.DataStorage.events.assessmentReady);
		$(this).trigger(DKI.DataStorage.events.questionSelected, this.assessment.currentQuestion);
	}
	else {
		$(this).trigger(DKI.DataStorage.events.attemptsMaxed);
	}
};

/**
 * Creates an assessment review object for a given assessment, fires the appropriate events
 * @param assessment Object The assessment to review
 * @fires assessmentReady
 * @fires questionSelected
 */
DKI.DataStorage.prototype.createReview = function (assessment) {
	//testing to see if we are reviewing a review.
	if(assessment.test){
		assessment = assessment.test;
	}
	this.assessment = new DKI.AssessmentReview(assessment);
	$(this).trigger(DKI.DataStorage.events.assessmentReady);
	$(this).trigger(DKI.DataStorage.events.questionSelected, this.assessment.currentQuestion);
};

/**
 * Takes a page ID and returns the corresponding page object
 * @param pageID String
 * @returns Object
 */
DKI.DataStorage.prototype.findPage = function (pageID) {
	var currentMod, currentObj;
	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		currentMod = this.courseStructure.modules[i];
		for (var j = 0; j < currentMod.objects.length; j ++) {
			currentObj = currentMod.objects[j];
			for (var k = 0; k < currentObj.pages.length; k ++) {
				if (currentObj.pages[k].id == pageID) {
					return currentObj.pages[k];
				}
			}
		}
	}
};

/**
 * Takes a subeoID and returns the corresponding page object
 * @param subeoId String
 * @returns Object
 */
DKI.DataStorage.prototype.getPageFromSubeo = function (subeoId) {	
	var currentMod, currentObj;
	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		currentMod = this.courseStructure.modules[i];
		for (var j = 0; j < currentMod.objects.length; j ++) {
			currentObj = currentMod.objects[j];
			for (var k = 0; k < currentObj.pages.length; k ++) {
				if (currentObj.pages[k].subeoid == subeoId) {
					return currentObj.pages[k];
				}
			}
		}
	}
};

DKI.DataStorage.prototype.getModuleEnd = function (module) {
	var objInd;
	var pageInd;
	if (typeof module === "string" || typeof module === "number") {
		module = this.findModule(module);
	}

	objInd = module.objects.length - 1;
	pageInd = module.objects[objInd].pages.length - 1;

	return module.objects[objInd].pages[pageInd];


};

/**
 * Takes an objectID and returns the corresonping learning object
 * @param objectID String
 * @returns Object
 */
DKI.DataStorage.prototype.findObject = function (objectID) {
	var currentMod;

	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		currentMod = this.courseStructure.modules[i];
		for (var j = 0; j < currentMod.objects.length; j ++) {
			if (currentMod.objects[j].id == objectID) {
				return currentMod.objects[j];
			}
		}
	}
};

/**
 * Takes a moduleID and returns the corresponding module object
 * @param moduleID String
 * @returns Object
 */
DKI.DataStorage.prototype.findModule = function (moduleID) {
	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		if (this.courseStructure.modules[i].id == moduleID) {
			return this.courseStructure.modules[i];
		}
	}
};

/**
 * Takes a moduleID or module object, and return's it's index within a course
 * @param module String|Object
 * @returns Integer
 */
DKI.DataStorage.prototype.findModuleIndex = function (module) {
	if (typeof module === "object") {
		module = module.id;
	}
	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		if (this.courseStructure.modules[i].id == module) {
			return i;
		}
	}
};


/**
 * Returns the score of the current assessment 
 * @returns Integer
 */
DKI.DataStorage.prototype.getAssessmentScore = function () {
	if (this.assessment) {
		return this.assessment.getScore();
	}
	else {
		return null;
	}
};

/**
 * Returns the scoring details of the current assessment 
 * @returns Object
 */
DKI.DataStorage.prototype.getAssessmentScoringDetails = function () {
	if (this.assessment) {
		return {
			score: this.assessment.getScore(),
			points: this.assessment.getPoints(),
			weight: this.assessment.getWeight()
		};
	}
	else {
		return null;
	}
};

/**
 * Returns the percentage of the course currently completes, as a function of total pages
 * @returns Integer
 */
DKI.DataStorage.prototype.getCourseCompletion = function () {
	var pageCount = 0;
	var completePages = 0;
	var currentModule, currentObject;
	if (this.courseStructure.complete || (this.courseStructure.testOnly == "1" && this.allTestsAttempted())) {
		return 100;
	}
	else {
		for (var i = 0; i < this.courseStructure.modules.length; i ++) {
			currentModule = this.courseStructure.modules[i];
			for (var j = 0; j < currentModule.objects.length; j ++) {
				currentObject = currentModule.objects[j];
				pageCount += currentObject.pages.length;
				if (currentObject.complete || currentModule.complete) {
					completePages += currentObject.pages.length;
				}
				else {
					for (var k = 0; k < currentObject.pages.length; k ++) {
						if (currentObject.pages[k].complete) {
							completePages ++;
						}
					}
				}
			}
		}

		return Math.round((completePages / pageCount) * 100);
	}
};

/**
 * Returns the current score (as a percentage) for all course assessments
 * @returns Integer
 */
DKI.DataStorage.prototype.getCourseScore = function () {
	if (!this.hasTesting()) {
		return null;
	}
	else {
		return this.getCourseScoringDetails().score;
	}
};

/**
 * Returns the the details for all course assessments
 * @returns Object
 */
DKI.DataStorage.prototype.getCourseScoringDetails = function () {
	var points = parseFloat(0, "10");
	var weight = 0;
	var score = 0;
	var currentModDetails;
	for (var i = 0; i < this.courseStructure.modules.length; i++) {		
		currentModDetails = this.getModuleScoringDetails(this.courseStructure.modules[i]);
		weight += currentModDetails.weight;
		points += currentModDetails.points;
	}
	if (points > 0 && weight > 0) {
		score = parseInt((points / weight) * 100, 10);
	}
	else{
		score = 0;
	}
	return {
		score: score,
		weight: weight,
		points: points
	};
};

/**
 * Returns the current score (as a percentage) for all currently attempted course assessments
 * @returns Integer
 */
DKI.DataStorage.prototype.getCourseScoreAvg = function () {
	if (this.hasTesting) {
		return this.getCourseScoringDetails().score;
	}
	else {
		return null;
	}
};

/**
 * Returns the current question
 * @return Object
 */
DKI.DataStorage.prototype.getCurrentQuestion = function () {
	if (this.assessment && this.assessment.currentQuestion) {
		return this.assessment.currentQuestion;
	}
};

DKI.DataStorage.prototype.getNextQuestion = function () {
	if (this.assessment) {
		return this.assessment.nextQuestion;
	}
};

DKI.DataStorage.prototype.getPreviousQuestion = function () {
	if (this.assessment) {
		return this.assessment.previousQuestion;
	}
};

/**
 * Returns the current question number
 * @return Integer
 */
DKI.DataStorage.prototype.getCurrentQuestionCount = function () {
	if (this.assessment) {
		return this.assessment.currentQuestionCount;
	}
};

/**
 * Calculates/returns the percentage of the moudle currently complete, as a function of the total number of pages
 * @param module Object|Integer|String The module to calculate completion for
 * @returns Integer Between 0 and 100 (inclusive)
 */
DKI.DataStorage.prototype.getModuleCompletion = function (module) {
	var pageCount = 0;
	var completePages = 0;
	if (typeof module === "number" || typeof module === "string") {
		module = this.findModule(module);
	}

	if (module.complete) {
		return 100;
	}
	else {
		for (var i = 0; i < module.objects.length; i ++) {
			pageCount += module.objects[i].pages.length;
			if (module.objects[i].complete) {
				completePages += module.objects[i].pages.length;
			}
			else {
				for (var j = 0; j < module.objects[i].pages.length; j ++) {
					if (module.objects[i].pages[j].complete) {
						completePages ++;
					}
				}
			}
		}

		return Math.round((completePages / pageCount) * 100);
	}
};

/**
 * Returns the score for the module.  If both a pre and post test are recorded, returns whichever is larger
 * @param module Object|Integer|String
 * @returns Integer
 */
DKI.DataStorage.prototype.getModuleScore = function (module) {
	var score = null;
	if (typeof module === "number" || typeof module === "string") {
		//Simple value, get the module object
		module = this.findModule(module);
	}
	if (module.pre) {
		score = module.pre;
	}
	if (module.post) {
		if (score && module.post.score > score || !score) {
			score = module.post.score;
		}
	}
	
	return score;
};

/**
 * Returns the scoring details for the module.  If both a pre and post test are recorded, returns whichever score is larger
 * @param module Object|Integer|String
 * @returns Object
 */
DKI.DataStorage.prototype.getModuleScoringDetails = function (module) {
	var score = null;
	var points = null;
	var weight = null;
	var passed = null;
	if (typeof module === "number" || typeof module === "string") {
		//Simple value, get the module object
		module = this.findModule(module);
	}	
	if (module.pre) {
		score = module.pre.score;
		points = module.pre.points;
		weight = module.pre.weight;
	}
	if (module.post) {
		if (score && module.post.score > score || !score) {
			score = module.post.score;
			points = module.post.points;
			weight = module.post.weight;
		}
	}

	if (module.passMark) {
		passed = score >= module.passMark;
	}
	else {
		passed = score >= this.passMark;
	}
	
	return {
		score: score,
		points: points,
		weight: weight,
		passed: passed
	};
};

/**
 * Returns the module that precedes the current module
 * @return Object
 */
DKI.DataStorage.prototype.getPreviousModule = function () {
	if (this.currentModuleIndex > 0) {
		return this.courseStructure.modules[this.currentModuleIndex - 1];
	}
};

/**
 * Returns the page the precedes the current page
 * @return Object
 */
DKI.DataStorage.prototype.getPrecedingPage = function (pageID) {
	var pageIndex, objectIndex, moduleIndex;
	if (pageID) {
		var indices = this.locatePage(pageID);
		pageIndex = indices.page;
		objectIndex = indices.object;
		moduleIndex = indices.module;
	}
	else {
		pageIndex = this.currentPageIndex;
		objectIndex = this.currentObjecIndex;
		moduleIndex = this.currentModuleIndex;
	}

	if (pageIndex > 0) {
		return this.courseStructure.modules[moduleIndex].objects[objectIndex].pages[pageIndex - 1];
	}
	else if (objectIndex > 0) {
		objectIndex --;
		pageIndex = this.courseStructure.modules[moduleIndex].objects[objectIndex].pages.length - 1;
		return this.courseStructure.modules[moduleIndex].objects[objectIndex].pages[pageIndex];
	}
	else if (moduleIndex > 0) {
		moduleIndex --;
		objectIndex = this.courseStructure.modules[moduleIndex].objects.length - 1;
		pageIndex = this.courseStructure.modules[moduleIndex].objects[objectIndex].pages.length - 1;
		return this.courseStructure.modules[moduleIndex].objects[objectIndex].pages[pageIndex];
	}
	else {
		return null;
	}
};

/**
 * Returns the total number of question for the current assessment
 * @return Integer
 */
DKI.DataStorage.prototype.getTotalQuestions = function () {
	if (this.assessment) {
		return this.assessment.totalQuestions;
	}
};
/**
 * Returns the the page immediately after the current one, or null of current page is 
 * end of course
 *
 * @return Object
 */
DKI.DataStorage.prototype.getNextPage = function () {
	var nextObject;
	var nextModule;
	if (this.currentPageIndex < (this.currentPageCount - 1)) {
		return this.currentObject.pages[this.currentPageIndex + 1];
	}
	else if (this.currentObjectIndex < (this.currentObjectCount -1)) {
		nextObject = this.currentModule.objects[this.currentObjectIndex + 1];
		return nextObject.pages[0];
	}
	else if (this.currentModuleIndex < (this.moduleCount - 1)) {
		nextModule = this.courseStructure.modules[this.currentModuleIndex + 1];
		nextObject = nextModule.objects[0];
		return nextObject.pages[0];
	}
	else {
		return null;
	}
};

/**
 * Returns the the page immediately before the current one, or null of current page is 
 * beginning of course
 *
 * @return Object
 */
DKI.DataStorage.prototype.getPreviousPage = function () {
	var pageCount;
	var objectCount;
	var objectLength;
	var prevObject;
	var prevModule;

	if (this.currentPageIndex > 0) {
		return this.currentObject.pages[this.currentPageIndex - 1];
	}
	else if (this.currentObjectIndex > 0) {
		prevObject = this.currentModule.objects[this.currentObjectIndex - 1];
		pageCount = prevObject.pages.length;

		return prevObject.pages[pageCount - 1]; 
	}
	else if (this.currentModuleIndex > 0) {
		prevModule = this.courseStructure.modules[this.currentModuleIndex - 1]; 
		objectCount = prevModule.objects.length;
		prevObject = prevModule.objects[objectCount - 1];
		pageCount = prevObject.pages.length;

		return prevObject.pages[pageCount - 1];
	}
	else {
		return null;
	}

};

/**
 * Advances to the next page in the course, adjusting current object/module if necessary
 * @fires pageSelected
 */
DKI.DataStorage.prototype.goNextPage = function () {
	if (this.currentPageIndex < (this.currentPageCount - 1)) {
		this.currentPageIndex ++;
		this.currentPage = this.currentObject.pages[this.currentPageIndex];
		this.currentModulePage ++;
		$(this).trigger(DKI.DataStorage.events.changePage, "forward");
		$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
	}
	else {
		this.goNextObject();
	}
	this.previousPage = this.getPreviousPage();
	this.nextPage = this.getNextPage();
};

/**
 * Goes to the previous page in the course, adjusting current object/module if necessary
 * @fires pageSelected
 */
DKI.DataStorage.prototype.goPreviousPage = function () {
	if (this.currentPageIndex > 0) {
		this.currentPageIndex --;
		this.currentPage = this.currentObject.pages[this.currentPageIndex];
		this.currentModulePage --;
		$(this).trigger(DKI.DataStorage.events.changePage, "back");
		$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
	}
	else {
		this.goPreviousObject();
	}
	this.previousPage = this.getPreviousPage();
	this.nextPage = this.getNextPage();
};

/**
 * Goes to the next learning object in the course, adjusting current page/module as necessary
 * @fires changeObject
 * @fires pageSelected
 */
DKI.DataStorage.prototype.goNextObject = function () {
	if (this.currentObjectIndex < (this.currentObjectCount - 1)) {
		this.currentObjectIndex ++;
		this.currentPageIndex = 0;
		this.currentObject = this.currentModule.objects[this.currentObjectIndex];
		this.currentPage = this.currentObject.pages[this.currentPageIndex];
		this.currentModulePage ++;
		this.currentPageCount = this.currentObject.pages.length;
		$(this).trigger(DKI.DataStorage.events.changeObject, "forward");
		$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
	}
	else {
		this.goNextModule();
	}
};

/**
 * Goes to the previous learning object in the course, adjusting current page/module as necessary
 * @fires changeObject
 * @fires pageSelected
 */
DKI.DataStorage.prototype.goPreviousObject = function () {
	if (this.currentObjectIndex > 0) {
		this.currentObjectIndex --;
		this.currentObject = this.currentModule.objects[this.currentObjectIndex];
		this.currentModulePage --;
		this.currentPageCount = this.currentObject.pages.length;
		this.currentPageIndex = this.currentPageCount - 1;
		this.currentPage = this.currentObject.pages[this.currentPageIndex];
		$(this).trigger(DKI.DataStorage.events.changeObject, "back");
		$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
	}
	else {
		this.goPreviousModule();
	}
};

/**
 * Goes to the next module in the course, 
 * @fires changeModule
 * @fires pageSelected
 * @fires courseEnd (if current module was the last in the course)
 */
DKI.DataStorage.prototype.goNextModule = function () {
	var hasTesting = false;
	if (this.currentModuleIndex < (this.moduleCount - 1)) {
		this.currentModuleIndex++;	
		this.currentObjectIndex = 0;
		this.currentPageIndex = 0;
		this.currentModule = this.courseStructure.modules[this.currentModuleIndex];
		this.currentObject = this.currentModule.objects[this.currentObjectIndex];
		this.currentPage = this.currentObject.pages[this.currentPageIndex];
		this.currentObjectCount = this.currentModule.objects.length;
		this.currentPageCount = this.currentObject.pages.length;
		this.currentModulePage = 1;
		this.currentModulePageTotal = this.countModulePages(this.currentModule);
		$(this).trigger(DKI.DataStorage.events.changeModule, "forward");
		if (this.currentPage) {
			$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
		}

		return true;
	}else {
		if (this.currentModuleIndex == this.courseStructure.modules.length-1 
			&& this.moduleHasTest(this.currentModule)){
			hasTesting = true;
			this.currentModuleIndex += 1;
		}
 
		$(this).trigger(DKI.DataStorage.events.courseEnd, hasTesting);
		return false;
	}
};

/**
 * Returns the next module, if there is one.  NULL otherwise
 * @returns [Object|null]
 */
DKI.DataStorage.prototype.getNextModule = function (module) {
	var modIndex = this.currentModuleIndex;

	if (module) {
		modIndex = this.findModuleIndex(module);
	}

	if (modIndex < (this.moduleCount - 1)) {
		return this.courseStructure.modules[modIndex + 1];
	}
	else {
		return null;
	}
};

/**
 * Goes to the previous module in the course, adjusting current object/page.  Fires appropriate events
 * @fires changeModule If current module was not the first in the course
 * @fires pageSelected If current module was not the first in the course
 */
DKI.DataStorage.prototype.goPreviousModule = function () {
	if (this.currentModuleIndex > 0) {
		this.currentModuleIndex --;
		this.currentModule = this.courseStructure.modules[this.currentModuleIndex];
		this.currentModulePageTotal = this.countModulePages(this.currentModule);
		this.currentModulePage = this.currentModulePageTotal;
		this.currentObjectCount = this.currentModule.objects.length;
		this.currentObjectIndex = this.currentObjectCount - 1;
		this.currentObject = this.currentModule.objects[this.currentObjectIndex];
		this.currentPageCount = this.currentObject.pages.length;
		this.currentPageIndex = this.currentPageCount - 1;
		this.currentPage = this.currentObject.pages[this.currentPageIndex];
		$(this).trigger(DKI.DataStorage.events.changeModule, "back");
		$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
	}
};

/**
 * Goes to the next question in the current assessment, fires appropriate event if assessment is completed
 * @fires questionSelected If there is another question to display
 * @fires assessmentCompleted If the last question in the assessment was completed
 */
DKI.DataStorage.prototype.goNextQuestion = function () {
	if (this.assessment) {
		this.assessment.goNextQuestion();
	}

	if (this.assessment.currentQuestion) {
		$(this).trigger(DKI.DataStorage.events.questionSelected, this.assessment.currentQuestion);
	}
	else {
		//Need to set the actual assessment for the module end screen
		if (this.assessment.test != null && this.assessment.test != undefined) {
			//This.assessment is actually an assessment review
			this.assessment = this.assessment.test
		}
		$(this).trigger(DKI.DataStorage.events.assessmentCompleted);
	}
};

/**
 * Goes to the previous question in the assessment, fires appropriate events
 * @fires questionSelected If there is a question to display
 * @fires assessmentCompleted If the assessment was finished
 */
DKI.DataStorage.prototype.goPreviousQuestion = function () {
	if (this.assessment) {
		this.assessment.goPreviousQuestion();
	}

	if (this.assessment.currentQuestion) {
		$(this).trigger(DKI.DataStorage.events.questionSelected, this.assessment.currentQuestion);
	}
	else {
		//Need to set the actual assessment for the module end screen
		if (this.assessment.test != null && this.assessment.test != undefined) {
			//This.assessment is actually an assessment review
			this.assessment = this.assessment.test
		}
		$(this).trigger(DKI.DataStorage.events.assessmentCompleted);
	}
};

/**
 * Returns true if the course has any testing associated with it
 * @return Boolean
 */
DKI.DataStorage.prototype.hasTesting = function () {
	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		if (this.courseStructure.modules[i].weight > 0) {
			return true;
		}
	}

	return false;
};

/**
 * Initializes event handlers
 */
DKI.DataStorage.prototype.initEvents = function () {
	var that = this;
	$(this).bind(DKI.DataStorage.events.courseCompleted, function () {
		that.onCourseComplete();
	});
	$(this).on(DKI.DataStorage.events.assessmentCompleted, function (e) {
		that.onAssessmentComplete(e);
	});
};

/**
 * Initializes the course structure.  Sets module weight to 1 or 0 depending on whether the module has any
 * testing associated
 * @param courseStructure Object
 */
DKI.DataStorage.prototype.initStructure = function (courseStructure) {
	for (var i = 0; i < courseStructure.modules.length; i ++) {
		if (this.moduleHasTest(courseStructure.modules[i])) {
			courseStructure.modules[i].weight = 1;
		}
		else {
			courseStructure.modules[i].weight = 0;
		}
	}

	return courseStructure;
};

/**
 * Returns true if the current page is the last of it's module, false otherwise.
 * @return Boolean
 */
DKI.DataStorage.prototype.isEndOfModule = function () {
	return (this.currentModulePage === this.currentModulePageTotal);
};

/**
 * Returns true if the current page is the last in the course, false otherwise.
 */
DKI.DataStorage.prototype.isEndOfCourse = function () {
	return (this.isEndOfModule() && this.currentModuleIndex === (this.moduleCount - 1));
};

/**
 * Returns true if the given module's tests are maxed out
 * @param module The module to test
 * @return Boolean
 */
DKI.DataStorage.prototype.isMaxedOut = function (module) {
	var allowedAttempts = 1;
	if (typeof module === "string" || typeof module === "number") {
		module = this.findModule(module);
	}
	if (this.behaviour.allowRetesting) {
		allowedAttempts += parseInt(this.behaviour.retestAttempts, "10");
	}
	return !(!module.post || module.post.attempts < allowedAttempts);
};

/**
 * Returns true if all the criteria for a passing status are met
 * @return Boolean
 */
DKI.DataStorage.prototype.isPassed = function (scoringDetails) {
	if (!scoringDetails) {
		scoringDetails = this.getCourseScoringDetails();
	}
	var passed = false;
	var modulesPassed = true;
	var moduleDetails;
	if (scoringDetails.score >= this.passMark && this.allTestsAttempted()) {
		if (this.behaviour.passModules) {
			for (var i = 0; i < this.courseStructure.modules.length; i += 1) {
				moduleDetails = this.getModuleScoringDetails(this.courseStructure.modules[i]);	
				if (this.courseStructure.modules[i].weight > 0 && !moduleDetails.passed) {
					modulesPassed = false;
					break;
				}
			}
			passed = modulesPassed;
		}
		else {
			passed = true;
		}
	}
	return passed;
};

/**
 * Returns true if all the criteria for a failed status are met
 * @return Boolean
 */
DKI.DataStorage.prototype.isFailed = function (scoringDetails) {
	if (!scoringDetails) {
		scoringDetails = this.getCourseScoringDetails();
	}
	return scoringDetails.score < this.passMark && this.testMaxedOut() 
};

DKI.DataStorage.prototype.jumpToModule =
function(module) {
	if (typeof module !== "object") {
		module = this.findModule(module);
	}
	if (!module) {
		return false;
	}
	this.currentModule = module;
	this.currentModuleIndex = this.findModuleIndex(module);
	if (this.courseStructure.testOnly !== "1") {
		this.currentObject = module.objects[0];
		this.currentObjectIndex = 0;
		this.currentPage = this.currentObject.pages[0];
		this.currentPageIndex = 0;
		this.currentModulePageTotal = this.countModulePages(this.currentModule);
		this.currentModulePage = 1;
		this.currentObjectCount = this.currentModule.objects.length;
		this.currentPageCount = this.currentObject.pages.length;
		this.previousPage = this.getPreviousPage();
		this.nextPage = this.getNextPage();
		$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
	}
	else {
		this.currentObject = null;
		this.currentObjectIndex = null;
		this.currentPage = null;
		this.currentPageIndex = null;
	}

	$(this).trigger(DKI.DataStorage.events.changeModule, "jump");
	return true;
	
};
/**
 * Jumps directly to a given page
 * @param pageID String
 */
DKI.DataStorage.prototype.jumpToPage = function (page) {
	var currentMod, currentObj;	
	var currentScreen = 0;
	var pageID = typeof page === "object" ? page.id : page;

	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		currentMod = this.courseStructure.modules[i];
		currentScreen = 0;
		for (var j = 0; j < currentMod.objects.length; j ++) {
			currentObj = currentMod.objects[j];
			for (var k = 0; k < currentObj.pages.length; k ++) {
				currentScreen ++;
				if (currentObj.pages[k].id == pageID) {
					this.currentPageIndex = k;
					this.currentPage = currentObj.pages[k];
					this.currentObjectIndex = j;
					this.currentObject = currentObj;
					this.currentModuleIndex = i;
					this.currentModule = currentMod;
					this.currentModulePageTotal = this.countModulePages(this.currentModule);
					this.currentModulePage = currentScreen;
					this.currentObjectCount = this.currentModule.objects.length;
					this.currentPageCount = this.currentObject.pages.length;
					this.previousPage = this.getPreviousPage();
					this.nextPage = this.getNextPage();
					$(this).trigger(DKI.DataStorage.events.changeModule, "jump");
					$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
					return true;
				}
			}
		}
	}
};

/**
 * Loads the stored completion information, if there is any
 * @fires pageSelected
 * @fires ready
 */
DKI.DataStorage.prototype.loadCompletion = function (autoResume) {
	//TODO: Load last location from local storage, if supported in the current browser
	this.initEvents();
	$(this).trigger(DKI.DataStorage.events.ready);
};

/**
 * Sets all pages preceding the given to 'complete'
 * @param page Integer | String | Object
 */
DKI.DataStorage.prototype.completePreviousPages = function (page) {
	if (typeof page === "string" || typeof page === "number") {
		page = this.findPage(page);
	}	
	var currentMod, currentObj;
	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		currentMod = this.courseStructure.modules[i];
		for (var j = 0; j < currentMod.objects.length; j ++) {
			currentObj = currentMod.objects[j];
			for (var k = 0; k < currentObj.pages.length; k ++) {
				if (currentObj.pages[k].id != page.id) {
					this.markPageComplete(currentObj.pages[k]);
				}
				else{
					return null;
				}
			}
			this.markObjectComplete(currentObj);
		}
		this.markModuleComplete(currentMod);		
	}
};

/**
 * Sets a page to complete, will set the parent object to complete if appropriate
 * @param page Integer | String | Object
 * @fire pageCompleted
 */
DKI.DataStorage.prototype.markPageComplete = function (page) {
	var parentObject;
	if (typeof page === "string" || typeof page === "number") {
		page = this.findPage(page);
	}
	this.doSetPageComplete(page);
	page.complete = true;
	parentObject = this.findObject(page.objectid);
	parentObject.started = true;
	$(this).trigger(DKI.DataStorage.events.pageCompleted, page.id);
	this.checkObjectComplete(parentObject);
};

/**
 * Sets an object to complete, will set the parent module to complete if appropriate
 * @param object Integer | String | Object
 * @fire objectCompleted
 */
DKI.DataStorage.prototype.markObjectComplete = function (object) {
	var parentModule;
	if (typeof object === "string" || typeof object === "number") {
		object = this.findObject(object);
	}
	this.doSetObjectComplete(object);
	parentModule = this.findModule(object.moduleid);
	parentModule.started = true;
	$(this).trigger(DKI.DataStorage.events.objectCompleted, object.id);
	this.checkModuleComplete(this.findModule(object.moduleid));
};

/**
 * Sets a module to complete, will set course completion if necessary
 * @param module Integer | String | Object
 * @fire moduleCompleted
 */
DKI.DataStorage.prototype.markModuleComplete = function (module) {
	if (typeof module === "string" || typeof module === "number") {
		module = this.findModule(module);
	}
	this.doSetModuleComplete(module);
	$(this).trigger(DKI.DataStorage.events.moduleCompleted, module.id);
	this.checkCourseComplete();
};

/**
 * Returns true if the given module has testing
 * @param module Object
 * @returns Boolean
 */
DKI.DataStorage.prototype.moduleHasTest = function (module) {
	for (var i = 0; i < module.objects.length; i ++) {
		if (module.objects[i].randQuest !== 0 && module.objects[i].questions.length > 0) {
			return true;
		}
	}

	return false;
};

/**
 * Returns array of all modules that have had testing (pre or post) attempted
 * @returns Object[] Array of module objects that have had testing attempted.
 */
DKI.DataStorage.prototype.modulesTestAttempted = function () {
	var mods = this.modulesWithTesting();
	var attempted = [];
	for (var i = 0; i < mods.length; i ++) {
		if (typeof(mods[i].pre) == "object" || typeof(mods[i].post) == "object") {
			attempted[attempted.length] = mods[i];
		}
	}

	return attempted;
};

/**
 * Returns array of module objects in the course that have testing
 * @returns Object[]
 */
DKI.DataStorage.prototype.modulesWithTesting = function () {
	var mods = [];
	for (var i = 0; i < this.courseStructure.modules.length; i++) {
		if (this.courseStructure.modules[i].weight > 0) {
			mods[mods.length] = this.courseStructure.modules[i];
		}
	}

	return mods;
};

/**
 * Returns the number of remaining preTest attempts for a given module
 * @param module Integer | String | Object
 * @returns Integer
 */
DKI.DataStorage.prototype.remainingPreAttempts = function (module) {
	if (!this.behaviour.enablePreTest) {
		return 0;
	}
	else {
		if (typeof module === "string" || typeof module === "number") {
			module = this.findModule(module);
		}
		if (module.weight === 0) {
			return 0;
		}
		if (module.pre || this.getModuleCompletion(module) > 0) {
			return 0;
		}
	}

	return 1;
};

/**
 * Returns the number of remaining postTest attempts for a given module
 * @param module Integer | String | Object
 * @returns Integer
 */
DKI.DataStorage.prototype.remainingPostAttempts = function (module) {
	var allowedAttempts;
	var takenAttempts = 0;
	if (!this.behaviour.enablePostTest) {
		return 0;
	}
	else {
		if (typeof module === "string" || typeof module === "number") {
			module = this.findModule(module);
		}
		if (module.weight === 0) {
			return 0;
		}
		if (!this.behaviour.allowRetesting) {
			allowedAttempts = 1;
		}
		else {
			allowedAttempts = parseInt(this.behaviour.retestAttempts, 10) + 1;
		}
		if (module.post) {
			takenAttempts = module.post.attempts;
		}

		allowedAttempts = allowedAttempts - takenAttempts;

		if (allowedAttempts < 0) {
			allowedAttempts = 0;
		}

		return allowedAttempts;
	}
};

/**
 * When player is intialized, this call will jump to the 
 */
DKI.DataStorage.prototype.resume = function (autoResume) {
	if (this.bookmark && (autoResume === "on" || (autoResume === "prompt" && confirm(this.strings.iResume)))) {
		this.jumpToPage(this.bookmark);
	}
	else {
		$(this).trigger(DKI.DataStorage.events.pageSelected, this.currentPage);
	}
};

/**
 * Sets the current page to complete
 * @calls markPageComplete
 */
DKI.DataStorage.prototype.setCurrentPageComplete = function () {
	this.markPageComplete(this.currentPage);
};

/**
 * Sets the current object to complete
 * @calls markObjectComplete
 */
DKI.DataStorage.prototype.setCurrentObjectComplete = function () {
	this.markObjectComplete(this.currentObject);
};

/**
 * Sets a page to complete
 * @param pageID String
 * @calls markPageComplete
 */
DKI.DataStorage.prototype.setPageComplete = function (pageID) {
	var page = this.findPage(pageID);
	if (page) {
		this.markPageComplete(page);
	}
};

/**
 * Sets an object to complete
 * @param objectID String
 * @calls markObjectComplete
 */
DKI.DataStorage.prototype.setObjectComplete = function (objectID) {
	var object = this.findObject(objectID);
	if (object) {
		this.markObjectComplete(object);
	}
};

/**
 * Sets a module to complete
 * @param moduleID
 * @calls markModuleComplete
 */
DKI.DataStorage.prototype.setModuleComplete = function (moduleID) {
	var module = this.findModule(moduleID);
	if (module) {
		this.markModuleComplete(module);
	}
}; 

/**
 * Submits the current question, for the current assessment
 * @param questionObject Object {id, options}
 */
DKI.DataStorage.prototype.submitCurrentQuestion = function (questionObject) {
	if (this.assessment) {
		var question = this.assessment.currentQuestion;
		var score = DKI.Marker.scoreQuestion(question, questionObject.options);
		this.assessment.scoreQuestion(question, questionObject, score);
		$(this).trigger(DKI.DataStorage.events.questionSubmitted, score);
		if (this.assessment.currentQuestion) {
			$(this).trigger(DKI.DataStorage.events.questionSelected, this.assessment.currentQuestion);
		}
		else {
			if (this.assessment.prepost === "pre") {
				this.assessment.module.pre.score = this.assessment.getScore();
				this.assessment.module.pre.points = parseFloat(this.assessment.getPoints(), "10");
				this.assessment.module.pre.weight = this.assessment.getWeight();
			}
			else {
				this.assessment.module.post.score = this.assessment.getScore();
				this.assessment.module.post.points = parseFloat(this.assessment.getPoints(), "10");
				this.assessment.module.post.weight = this.assessment.getWeight();
			}
			$(this).trigger(DKI.DataStorage.events.assessmentCompleted);
		}
	}
};

/**
 * Returns true if all test attempts are maxed out
 * @returns true
 */
DKI.DataStorage.prototype.testMaxedOut = function () {
	var totalTests = 0;
	var completedTests = 0;
	var allowedAttempts = 1;
	if (!this.hasTesting() || !(this.behaviour.enablePreTest || this.behaviour.enablePostTest)) {
		return true;
	}
	else if (this.hasTesting()) {
		if (this.behaviour.enablePreTest && !this.behaviour.enablePostTest) {
			for (var i = 0; i < this.courseStructure.modules.length; i ++) {
				if (this.courseStructure.modules[i].weight > 0) {
					totalTests ++;
					if (this.courseStructure.modules[i].pre || this.getModuleCompletion(this.courseStructure.modules[i]) > 0) {
						completedTests ++;
					}
				}
			}
		}
		else {
			if (this.behaviour.allowRetesting) {
				allowedAttempts += parseInt(this.behaviour.retestAttempts, 10);
			}
			for (var i = 0; i < this.courseStructure.modules.length; i ++) {
				if (this.courseStructure.modules[i].weight > 0) {
					totalTests ++;
					if (this.courseStructure.modules[i].post && this.courseStructure.modules[i].post.attempts >= allowedAttempts) {
						completedTests ++;
					}
				}
			}
		}
	}

	return totalTests === completedTests;
};

// 'Helper' methods, not intended to be called outside of the object
/**
 * Returns an object representing the test questions
 * @returns object
 */
DKI.DataStorage.prototype.loadTest = function (module) {
	var testObject = {questions: []};
	var moduleObjects = module.objects.slice();
	var objectCount = moduleObjects.length;
	var randIndex;
	//randomize the LO's for the module
	for (var i = 0; i < objectCount; i ++) {
		randIndex = Math.floor(Math.random() * moduleObjects.length);
		testObject.questions = testObject.questions.concat(this.randomQuestions(moduleObjects.splice(randIndex, 1)[0]));	
	}

	return testObject;
};

DKI.DataStorage.prototype.randomQuestions = function (object) {
	var questions = [];
	var sourceQuestions = object.questions.slice();
	var randIndex;

	if (object.randQuest >= sourceQuestions.length && parseInt(object.preserveQuestionOrder, 10) === 1) {
		return sourceQuestions;
	}
	else {
		for (var i = 1; i <= object.randQuest; i ++) {
			if (sourceQuestions.length > 0) {
				randIndex = Math.floor(Math.random() * sourceQuestions.length);
				questions[questions.length] = sourceQuestions.splice(randIndex, 1)[0];
			}
			else {
				return questions;
			}
		}
	}

	return questions;
};

DKI.DataStorage.prototype.locatePage = function (pageID) {
	var indices = {
		page: 0,
		object: 0,
		module: 0
	};
	var currentMod, currentObject;

	for (var i = 0; i < this.courseStructure.modules.length; i ++) {
		currentMod = this.courseStructure.modules[i];
		for (var j = 0; j < currentMod.objects.length; j ++) {
			currentObject = currentMod.objects[j];
			for (var k = 0; k < currentObject.pages.length; k ++) {
				if (currentObject.pages[k].id === pageID) {
					indices.page = k;
					indices.object = j;
					indices.module = i;
					return indices;
				}
			}
		}
	}

	return indices;
};

/**
 * Marks the course and all descendants as complete
 */
DKI.DataStorage.prototype.doSetCourseComplete = function () {
	this.courseStructure.complete = true;
	this.courseStructure.started = true;
	for (var i = 0; i < this.courseStructure.modules.length; i += 1) {
		this.doSetModuleComplete(this.courseStructure.modules[i]);
	}
};

/**
 * Marks a modules and all it's descendants as complete
 * @param module Object
 */
DKI.DataStorage.prototype.doSetModuleComplete = function (module) {
	module.complete = true;
	this.doSetModuleStarted(module);
	for (var i = 0; i < module.objects.length; i += 1) {
		this.doSetObjectComplete(module.objects[i]);
	}
};

/**
 * Marks module and course as started
 * @param module Object
 */
DKI.DataStorage.prototype.doSetModuleStarted = function (module) {
	module.started = true;
	this.courseStructure.started = true;
};

/**
 * Marks a learning object and all children as complete.
 * @param object Object
 */
DKI.DataStorage.prototype.doSetObjectComplete = function (object) {
	object.complete = true;
	this.doSetObjectStarted(object);
	for (var i = 0; i < object.pages.length; i += 1) {
		this.doSetPageComplete(object.pages[i]);
	}
};

/**
 * Marks object and parent module as started
 * @param object Object
 */
DKI.DataStorage.prototype.doSetObjectStarted = function (object) {
	var parentModule = this.findModule(object.moduleid);
	object.started = true;
	this.doSetModuleStarted(parentModule);
};

/**
 * Marks page as started and complete.  
 * @param page Object
 */
DKI.DataStorage.prototype.doSetPageComplete = function (page) {
	var parentObject = this.findObject(page.objectid);
	page.complete = true;
	page.started = true;
	this.doSetObjectStarted(parentObject);
};

//Event handlers functions
//
/**
 * Handles the courseComplete event
 */
DKI.DataStorage.prototype.onCourseComplete = function () {
	var scoringDetails;
	if (this.hasTesting() && (this.behaviour.enablePreTest || this.behaviour.enablePostTest)) {
		scoringDetails = this.getCourseScoringDetails();
		if (this.isPassed(scoringDetails)) {
			this.lessonStatus = "passed";
		}
		else if (this.isFailed(scoringDetails)) {
			this.lessonStatus = "failed";
		}
	}
	else  {
		this.lessonStatus = "complete";
	}
};

/**
 * Handles the assessment Complete event
 */
DKI.DataStorage.prototype.onAssessmentComplete = function () {
	if (this.getCourseCompletion() === 100) {
		$(this).trigger(DKI.DataStorage.events.courseCompleted);
	}
};

DKI.DataStorage.events = {
	courseEnd: "courseEnd",
	testEnd: "testEnd",
	changePage: "changePage", //Fired whenever we go to next/previous page, but stay within the same object
	changeObject: "changeObject", //Fired whenever we go to next/previous object, but stay within the same module.  Provides a 'dir' argument, either 'forward' || 'back'.
	changeModule: "changeModule", //Fired whenever we go to the next/previous module.  Provides a 'dir' argument, either 'forward' || 'back'.
	pageCompleted: "pageCompleted", //Fired whenever a page is marked complete.  Provides the id of the page.
	objectCompleted: "objectCompleted", //Fired whenever an object is marked complete.  Provides the id of the object.
	moduleCompleted: "moduleCompleted", //Fired whenever a module is marked complete.  Provides the id of the module.
	courseCompleted: "courseCompleted", //Fired whenever the course is completed.
	pageSelected: "pageSelected", //Fired whenever the current page changes.  Provides the ID of the currently selected page.
	questionSelected: "questionSelected", //Fired whenever the current question changes.  Provides ID of the question's page.
	questionSubmitted: "questionSubmitted", //Fired whenever the current question is submitted.  Provides score of question.
	assessmentCompleted: "assessmentCompleted", //Fired whenever the current assessment is completed.
	assessmentReady: "assessmentReady", //Fired whenver a newly created asessment is in a ready state.
	attemptsMaxed: "attemptsMaxed", //Fired when attempting to create a test whose attempts are maxed out.
	ready: "ready" //Fired whenever datastorage is ready (previous completion/bookmark data has been loaded, player should now display the current page.
};
