var DKI;
if (!DKI) {
	DKI = {};
}
/**
 * Class representing the course player used to preview courses
 * @class PresentationPlayer
 * @namespace DKI
 * @constructor
 * @param cfg {Object} Configuration object.
 * @config dataStorage {Object} The dataStorage object which holds course structure and state.
 * @config behaviour {Object} The object which holds course behaviour options
 * @config settings {Object} The object which holds runtime settings, these change depending on how the course is launced.
 * @config strings {Object} The object holding strings to display to the user, depends on the course's language.
 * @uses DKI.Dom, YAHOO.util.Event, YAHOO.util.Connection, YAHOO.lang.JSON, swfobject
 */
DKI.PresentationPlayer = function (cfg) {
	/**
	 * The course data storage object
	 * @property dataStorage
	 * @type Object
	 */
	this.dataStorage = cfg.dataStorage;
	/**
	 * The raw course structure
	 * @property course
	 * @type Object
	 */
	this.course = cfg.dataStorage.courseStructure;
	/**
	 * Object containing the various player behaviour options
	 * @property behaviour
	 * @type Object
	 */
	this.behaviour = cfg.behaviour;
	this.settings = cfg.settings;
	this.strings = cfg.strings;
	this.rootURL = cfg.rootURL;
	if (Modernizr.csstransitions) {
		this.transitions = {
			page: this.dataStorage.courseStructure.pageTransition,
			jump: this.dataStorage.courseStructure.jumpTransition,
			end: this.dataStorage.courseStructure.endscreenTransition
		};
	}
	else {
		this.transitions = {
			page: "none",
			jump: "none",
			end: "none"
		};
	}
			
	/**
	 * The current page being displayed
	 * @property currentPage
	 * @type Object
	 */
	this.currentPage;
	this.contentPage = {
		previous: null,
		current: null,
		next: null
	};
	/**
	 * The current module being displayed
	 * @property currentModule
	 * @type Object
	 */
	this.currentModule;
	/**
	 * The current learning object being displayed
	 * @property currentObject
	 * @type Object
	 */
	this.currentObject;
	/**
	 * The current test being displayed
	 * @property currentTest
	 * @type Object
	 */
	this.currentTest;
	/**
	 * The index in the modules array of the current module
	 * @property currentModuleIndex
	 * @private
	 */
	this.currentModuleIndex = 0;
	/**
	 * The index in the learning objects array of the current module
	 * @property currentObjectIndex
	 * @private
	 */
	this.currentObjectIndex = 0;
	/**
	 * The index in the page array of the current page
	 * @property currentPageIndex
	 * @private
	 */
	this.currentPageIndex = 0;
	/**
	 * The frame used to display course content/questions
	 * @property contentFrame
	 * @type HTMLElement
	 * @private
	 */
	this.contentFrame = $("#contentFrame");
	/**
	 * Holds the 3 frames used to display page content
	 */
	this.pageFrames = {};
	this.pageFrames.previous = document.createElement("div");
	this.pageFrames.previous.className = "dkiContentFrame previous";
	this.pageFrames.current = document.createElement("div");
	this.pageFrames.current.className = "dkiContentFrame current";
	this.pageFrames.next = document.createElement("div");
	this.pageFrames.next.className = "dkiContentFrame next";
	this.contentFrame.append([
		this.pageFrames.previous, 
		this.pageFrames.current, 
		this.pageFrames.next
	]);
	/**
	 * The contentAPI object. Injected after player and contentAPI are instantiated
	 * @property contentAPI
	 * @type Object
	 */
	this.contentAPI;
	/**
	 * The course menu
	 * @property menu
	 * @type DKI.PlayerMenu
	 */
	this.menu = null;
	this.navigation = new DKI.PlayerNavigation(this);
	this.barHeight = $("#headerContainer").height() + $("#footerContainer").height();

	this.store = DKI.courseStore;	
	this.feedback = DKI.FeedbackPanel;
	this.feedbackTab = $("#feedbackTab");		// primary dom container for feedback button
	
	this.audio = true;
	this.isMobile = $(document.body).hasClass("mobile");
	this.isAdminPreview = $(document.body).hasClass("adminPreview");
	
	//State for assessments
	this.assessmentState;
	this.questionFeedbackClosed;
	this.reachedEnd = false;
	
	//Setup forward/back functions
	this.forward = this.contentForward;
	this.back = this.contentBack;
	this.reload = this.contentReload;

	this.events = {

		/**
		 * Triggered whenever the player loads a new page into the frame.
		 */
		pageLoaded: 'pageLoaded',
		/**
		 * Triggered whenever the player loads a content page into the frame.
		 */
		contentPageLoaded: 'contentPageLoaded',
		/**
		 * Triggered whenever the player loads a question page into the frame.
		 */
		questionPageLoaded: 'questionPageLoaded'
	};
	
	/**
	 * A boolean flag to determine if the course is currently in a test. Used
	 * by onFeedbackSubmit to add feedback notes to a question page instead of
	 * a regular page.
	 * @property inAssessment
	 * @type Boolean
	 */
	this.inAssessment = false;
	this.inEndScreen = false;
	
	/**
	 * A boolean flag to determin if the course is currently in a review.
	 * @property inReview
	 * @type Boolean
	 */
	this.inReview = false;
	/**
	 * Tracks loading/complete state of next/previous/current pages.  Used to prevent attempting to 'start'
	 * an unloaded page.
	 */
	this.loadState = {
		previous: "done",
		current: "done", 
		next: "done"
	};
	/**
	 * Queue any forward/back action calls
	 */
	this.queuedAction = null;
	//Apply this class to suppress page transitions.  Otherwise, we see the next page transition into place
	//when using the 'slide' transition
	this.contentFrame.addClass("loading");
	this.navigation.hideForwardButton();
	this.navigation.hideBackButton();
	$(this.pageFrames.previous).addClass(this.transitions.page);
	$(this.pageFrames.current).addClass(this.transitions.page);
	$(this.pageFrames.next).addClass(this.transitions.page);
	this.initEvents();
	this.loadCompletion();
	/**
	* Set up the modal
	*/
	$("#modalFrame").on("load", function(){		
		if(this.src == "about:blank") return;
		if(!$("#modalWrapper").data("top")){
			$("#modalWrapper").data("top", parseInt($("#modalWrapper").css("top").split("px")[0]));
			$("#modalWrapper").data("left", parseInt($("#modalWrapper").css("left").split("px")[0]));
		}
		var width = $("#contentFrame").width() - ($("#modalWrapper").data("left") * 2) - parseInt($("#modalWrapper").css("padding-left").split("px")[0]) - parseInt($("#modalWrapper").css("padding-right").split("px")[0]);
		var height = $("#contentFrame").height() - ($("#modalWrapper").data("top") * 2) - parseInt($("#modalWrapper").css("padding-top").split("px")[0]) - parseInt($("#modalWrapper").css("padding-bottom").split("px")[0]);
		var offset = $("#contentFrame").offset();		
		var top = offset.top + $("#modalWrapper").data("top");
		var left = offset.left + $("#modalWrapper").data("left") - (($("html").width() - $("body").width()) / 2);
		$("#modalWrapper").css({
			"top": top + "px",
			"left": left + "px",
			"width": width + "px",
			"height": height + "px"
		});
		
		$("#modalWrapper").fadeIn();
	});
	$("#modalWrapper .modalClose, #modalDisabler").on("click", function(){
		$("#modalWrapper, #modalDisabler").fadeOut();
	});
};


DKI.PresentationPlayer.prototype.applyTransitionClass = function (frame, newClass) {
	var pageFrame = $(this.pageFrames[frame]);
	pageFrame.removeClass(DKI.PresentationPlayer.transitionClasses);
	pageFrame.addClass(newClass);
};

/**
 * Initializes the events to be fired, as well as any event handlers
 */
DKI.PresentationPlayer.prototype.initEvents = function () {
	var that = this;

	if (this.behaviour.preventRightClick) {
		$(document).on("contextmenu", function () {return false;});
	}
	
	$(this).on(this.events.contentPageLoaded,function(){
		that.applyCssEffects.init()
	})
	
	$(this.dataStorage).on(DKI.DataStorage.events.ready, function (e) {
		that.initMenu();
		that.setPlayerSize();
		that.initFeedbackTab();
		that.resumeBookmark();
		//Remove the loading class, so page transitions will once again be seen
		that.contentFrame.removeClass("loading");
	});

	$(this.dataStorage).on(DKI.DataStorage.events.changeObject, function () {
		that.setObjectDisplay();
	});
	
	$(this.dataStorage).on(DKI.DataStorage.events.courseEnd, function (e, hasTesting) {
	});
	
	$(this.dataStorage).on(DKI.DataStorage.events.questionSubmitted, function (e, score, questionObject) {
		that.contentPage.current.setScore(score, questionObject);
	});
	
	$(this.dataStorage).on(DKI.DataStorage.events.assessmentReady, function () {
		that.initBarsAssessment();
		that.navigation.setCurrentScreen(that.dataStorage.getCurrentQuestionCount());
		that.loadCurrentQuestion();
		that.pageFrames.previous.innerHTML = "";
	});
	
	$(this.navigation).on(DKI.PlayerNavigation.events.forward, function () {
		that.forward();
	});
	$(this.navigation).on(DKI.PlayerNavigation.events.back, function () {
		that.back();
	});
	$(this.navigation).on(DKI.PlayerNavigation.events.reload, function () {that.reload();});	
	$(this.navigation).on(DKI.PlayerNavigation.events.menuToggle, function () {		
		that.toggleOutline();
	});
	$(this.navigation).on(DKI.PlayerNavigation.events.menuClose, function () {		
		that.hideTree();
	});
	$(this.navigation).on(DKI.PlayerNavigation.events.glossaryToggle, function (e, showGlossary) {
		if (showGlossary) {
			DKI.glossaryBrowse.show();
		}
		else {
			DKI.glossaryBrowse.hide();
		}
	});
	$(this.navigation).on(DKI.PlayerNavigation.events.exit, function () {
		if (scormAPI) {
			scormAPI.ConcedeControl();
		}
		else {
			if (that.behaviour.exitFunction != "") {
				eval(that.behaviour.exitFunction);
			}
			else{
				window.close();
			}
		}
	});
	$(this.navigation).on(DKI.PlayerNavigation.events.audioToggle, function (e, audio) {that.toggleAudio(audio);});
	$(this.navigation).on(DKI.PlayerNavigation.events.transcriptToggle, function () {
		that.toggleTranscript();
	});
	//Use the document object as an event pool
	//Handler for question submitted devent
	$(document).on(DKI.TestQuestion.events.submitted, function (e, submitObject) {
		//Disable forward/back
		that.navigation.hideForwardButton(); 
		that.navigation.hideBackButton();
		that.dataStorage.submitCurrentQuestion(submitObject);
	});
	//Handler for next button's click event
	$(document).on(DKI.TestQuestion.events.nextClicked, function () {
		that.forward();
	});
	//Handler for menu's page click event.
	$(document).on(DKI.PlayerMenu.events.pageClicked, function (e, pageid) {
		that.onPageClicked(pageid);
	});
	//Handler for page rezise event
	$(document).on(DKI.ContentPage.events.resize, function (e, size) {
		if (that.isMobile && that.settings.scrolling) {
			$(that.pageFrames.current).height(size);
			that.contentFrame.height(size);
			if(dkiUA.blackberry && parent && parent.setContentHeight){
				parent.setContentHeight(size);
			}
		}
	});
	//Bind to the event fired when a page is ready.
	$(document).on(DKI.ContentPage.events.ready, $.proxy(this.onPageReady, this));
	//Bind to the event fired when a module endscreen is ready.
	$(document).on(DKI.EndModule.events.ready, $.proxy(this.onEndReady, this));
};

/**
 * Initializes the player menu.
 */
DKI.PresentationPlayer.prototype.initMenu = function () {
	var self = this;
	this.menu = new DKI.PlayerMenu({
		element:"menu", 
		dataStorage: this.dataStorage, 
		behaviour: this.behaviour, 
		strings: this.strings,
		player: this
	});
	this.menuWidth = this.menu.getWidth();
	this.menuLeft = this.menu.getLeft();
	$(this.menu).on(DKI.PlayerMenu.events.closed, function () {self.hideTree();});
};

/**
 */
DKI.PresentationPlayer.prototype.checkPreTest = function () {
	if (this.behaviour.enablePreTest && 
			this.dataStorage.currentModule.weight > 0 && 
			this.dataStorage.getModuleCompletion(this.dataStorage.currentModule) === 0 && 
			typeof this.dataStorage.currentModule.pre !== "object") {

		if (this.behaviour.forcePreTest) {
			return true;
		}
		else {
			if (confirm(this.strings.pretestQuestLabel)) {
				return true;
			}
		}
	}

	return false;
};

/**
 * Displays the given URL in the content frame
 * @method displayURL
 * @param url {string} The URL to be displayed
 * @param frame {HTMLElement} The HTML element that should display the URL content
 * @param callback {Object | Function} Optional callback function(s).  If a function is provieded, 
 * executed on success.  Otherwise an object specifying any of success, error and complete functions
 */
DKI.PresentationPlayer.prototype.displayURL = function (url, frame, callback) {
	var that = this;
	var callBackObj = {
		success: function () {},
		error: function () {},
		complete: function () {}
	};
	if (callback) {
		if (typeof(callback) == "function") {
			callBackObj.success = callback;	
		}
		else {
			if (callback.success) {
				callBackObj.success = callback.success;
			}
			if (callback.error) {
				callBackObj.error = callback.error;
			}
			if (callback.complete) {
				callBackObj.complete = callback.complete;
			}
		}
	};
	$.get(url, function (data) {
		frame.innerHTML = data;
	}, "html").success(callBackObj.success).error(callBackObj.error).complete(callBackObj.complete);
};

DKI.PresentationPlayer.prototype.getCurrentTestScore = function () {
	return this.dataStorage.getAssessmentScore();
};

/**
 * Returns the number of pages in a module
 * @method getModuleScreenCount
 * @param module {Object} The module whose screen count is to be determined
 * @return {int} The number of screens (content pages) in the module
 */
DKI.PresentationPlayer.prototype.getModuleScreenCount = function (module) {
	var screenCount = 0;
	for (var i = 0; i < module.objects.length; i++) {
		screenCount += module.objects[i].pages.length;
	}

	return screenCount;
};

/**
 * Initialized the bars for learning content
 * @method initBars
 */
DKI.PresentationPlayer.prototype.initBars = function () {
	var page = this.dataStorage.currentPage;
	var prevPage = this.dataStorage.previousPage;
	this.navigation.contentMode();
	this.setModuleDisplay();
	this.setObjectDisplay();
	this.navigation.setCurrentScreen(this.dataStorage.currentModulePage);
	if (this.behaviour.transcripts) {
		if (page.transcript) {
			this.navigation.enableTranscriptButton();
		}
		else {
			this.navigation.disableTranscriptButton();
		}
	}
	else {
		this.navigation.hideTranscriptButton();
	}

	if (page.disableAudio) {
		this.navigation.hideAudioButton();
	}
	else {
		this.navigation.showAudioButton();
	}

	if (!prevPage) {
		this.navigation.hideBackButton();
	}
	if(DKI.glossaryBrowse.isEmpty()) {
		this.navigation.disableGlossaryButton();
	}
};

/**
 * Initializes the navigation bars for testing content
 * @method initBarsAssessment
 */
DKI.PresentationPlayer.prototype.initBarsAssessment = function () {	
	var showNav = false;
	if (this.inReview) {
		showNav = true;
	}
	this.navigation.testMode(showNav);
	this.navigation.setTotalScreens(this.dataStorage.getTotalQuestions());
	this.navigation.setCurrentScreen(this.dataStorage.getCurrentQuestionCount());
	this.navigation.hideBackButton();
};

/**
 * Initialized the navigation bars for end screens
 * @method initBarsEndScreen
 */
DKI.PresentationPlayer.prototype.initBarsEndScreen = function () {
	this.inEndScreen = true;
	if (this.behaviour.transcripts) {
		this.navigation.disableTranscriptButton();
	}
	this.navigation.endMode();
};

DKI.PresentationPlayer.prototype.initFeedbackTab = function () {
	var that = this;
	if (this.course.isDebug) {
		this.feedback.init(that,'feedbackPanel',DKI.courseStore, this.strings)
		this.feedbackTab.css("display", "block");
		this.feedbackTab.on(this.settings.clickEvent + " keypress", function (e) {
			e.stopPropagation();
			if (e.type !== "keypress" || (e.which === 13 || e.which == 32)) {
				that.feedback.toggleDisplay()
				return false;
			}
		});
	}
	return true;
};

/**
 * Jumps to a given page in the course
 * @method jumpTo
 * @param pageID {int} The ID of the page to jump to
 */
DKI.PresentationPlayer.prototype.jumpTo = function (pageID) {
	//TODO: Verify that page jump is allowed by checking enrollment options and completion status.
	
	this.dataStorage.jumpToPage(pageID);
	this.jumpComplete();
};

/**
 * Jumpts to a given module
 * @method jumpToModule
 * @param module {int|Object} The module to jump to
 */
DKI.PresentationPlayer.prototype.jumpToModule = function (module) {
	this.dataStorage.jumpToModule(module);
	this.jumpComplete();
};

/**
 * Executes logic to display content page after jumping
 */
DKI.PresentationPlayer.prototype.jumpComplete = function () {
	this.inAssessment = false;
	this.inEndScreen = false;
	this.contentAPI.hideBoth();
	this.initBars();
	//Ensure reachedEnd is false.  If we are jumping to the last page of the course, 
	//the 'loadCourseEnd' function call will set the value back to true
	this.reachedEnd = false;
	if (this.checkPreTest()) {
		this.launchTest(this.dataStorage.currentModule.id, "pre");
	}
	this.applyTransitionClass("next", this.transitions.jump);
	this.loadPage("next", this.dataStorage.currentPage, function () {
		this.dataStorage.setCurrentPageComplete();
		this.reload = this.contentReload;
		this.shuffleForward();
		this.pageFrames.next.innerHTML = "";
		this.pageFrames.previous.innerHTML = "";
		this.initBars();
		if (this.dataStorage.previousPage) {
			this.applyTransitionClass("previous", this.transitions.page);
			this.loadPage("previous", this.dataStorage.previousPage);
		}
		//Determine if an endscreen or page should be loaded in the 'next' slot
		if (this.dataStorage.isEndOfCourse()) {
			this.loadCourseEnd();
		}
		else if (!this.behaviour.skipEnd && this.dataStorage.isEndOfModule()) {
			this.loadModuleEnd();
		}
		else if (this.dataStorage.nextPage) {
			this.applyTransitionClass("next", this.transitions.page);
			this.loadPage("next", this.dataStorage.nextPage);
			//Ensure forward function executes next page, not a start endscreen function
			this.forward = this.contentForward;
		}
		this.back = this.contentBack;
	});
};
/**
 * Launches a review of an assessment
 * @method launchReview
 * @param assessment {object} The assessment to review
 */
DKI.PresentationPlayer.prototype.launchReview = function (assessment) {
	this.inReview = true;
	this.dataStorage.createReview(assessment);
	this.forward = this.assessmentForward;
	this.back = this.assessmentBack;
	this.reload = this.loadCurrentQuestion;
	this.hideTree();
	this.initBarsAssessment();
};

/**
 * Launches a test for a module
 * @method launchTest
 * @param loid {int} The ID of the module to launch a test for
 * @param prePost {int} If 1, launch a preTest.  If 2, launch a postTest
 */
DKI.PresentationPlayer.prototype.launchTest = function (loid, prePost) {
	this.inAssessment = true;
	this.dataStorage.createTest(loid, prePost);
	//Setup forward/back functions
	this.forward = this.assessmentForward;
	this.back = this.assessmentBack;
	this.reload = this.loadCurrentQuestion;
	this.hideTree();
};

DKI.PresentationPlayer.prototype.loadCertificate = function () {
	var that = this;
	this.displayURL(this.settings.certificateURL, this.pageFrames.current, function () {
		DKI.CompletionCertificate(that.contentAPI);
	});
};

/**
 * Loads the current completion information for the course and updates the menu
 * @method loadCompletion
 */
DKI.PresentationPlayer.prototype.loadCompletion = function () {
	this.dataStorage.loadCompletion(); //TODO: Use configured behaviour to determine if autoResume should be passed.
};

/**
 * Load the course end screen into the content frame
 * @method loadCourseEnd
 */
DKI.PresentationPlayer.prototype.loadCourseEnd = function () {
	var that = this;
	var loadModuleEnd = false;
	if (!this.reachedEnd && this.dataStorage.moduleHasTest(this.dataStorage.currentModule)) {
		loadModuleEnd = true;
	}
	this.reachedEnd = true;
	if (loadModuleEnd) {
		this.loadModuleEnd();
	}
	else {
		this.contentPage.next = DKI.EndCourse;
		this.applyTransitionClass("next", this.transitions.end);
		this.displayURL(this.settings.endCourseURL, this.pageFrames.next);
		DKI.EndCourse.init(this, this.pageFrames.next);
		this.forward = this.startCourseEnd;
	}	
};

/**
 * Load the current page content into the content frame
 * @method loadCurrentPage
 */
DKI.PresentationPlayer.prototype.loadCurrentPage = function () {
	var page = this.dataStorage.currentPage;
	this.forward = this.contentForward;
	this.back = this.contentBack;	
	//If pre-testing is enable, and the current module has a test & content has not been attempted
	//launch the pre-test
	if (this.checkPreTest()) {
		this.launchTest(this.dataStorage.currentModule.id, "pre");
	}
	else {
		this.applyTransitionClass("current", this.transitions.page);
		this.loadPage("current", page, function () {			
			this.reload = this.contentReload;
			this.contentPage.current.start();
			this.dataStorage.setCurrentPageComplete();
			this.navigation.setCurrentScreen(this.dataStorage.currentModulePage);

			this.initBars();	
			$(this).trigger(this.events.contentPageLoaded);
			$(this).trigger(this.events.pageLoaded);	
		});
		if (this.dataStorage.previousPage) {
			this.applyTransitionClass("previous", this.transitions.page);
			this.loadPage("previous", this.dataStorage.previousPage);
		}
		//Determine if an endscreen or page should be loaded in the 'next' slot
		if (this.dataStorage.isEndOfCourse()) {
			this.loadCourseEnd();
		}
		else if (!this.behaviour.skipEnd && this.dataStorage.isEndOfModule()) {
			this.loadModuleEnd();	
		}
		else if (this.dataStorage.nextPage) {
			this.applyTransitionClass("next", this.transitions.page);
			this.loadPage("next", this.dataStorage.nextPage);
		}
	}
};

/**
 * Loads a given content page, as either "previous", "current" or "next"
 * @method loadPage
 * @param position [String] One of "previous", "current" or "next".  Used to determine which pageFrames and contentPage
 * object key to use
 * @param page [Object] The page to load
 * @param callback [Function] Optional.  If provided, executed after page is loaded & instantiated
 */
DKI.PresentationPlayer.prototype.loadPage = function (position, page, callback) {
	$("#modalWrapper, #modalDisabler").hide();
	var that = this;
	var pageContainer = this.pageFrames[position]; 
	var pageMarkup = null;
	var pageActions = null;
	var initPage = function () {
		if (pageMarkup !== null && pageActions !== null) {
			that.contentPage[position] =  new DKI.ContentPage({
				contentAPI: that.contentAPI, 
				containerEl: pageContainer, 
				actions: pageActions,
				page: page,
				player: that
			});
			that.loadState[position] = "done";
			if (callback) {
				callback.call(that);
			}
			if (that.queuedAction) {
				//Next or previous have been called, execute after load is completed
				if (that.queuedAction === "forward" && position === "next") {
					that.forward();
				}
				else if (that.queuedAction === "resume" && position === "next") {
					that.resume();
				}
				else if (that.queuedAction === "back" && position === "previous") {
					that.back();
				}
				that.queuedAction = null;
			}
		}
	};
	this.loadState[position] = "loading";
	this.loadPageActions(page.id, function (data) {
		pageActions = data;
		initPage();
	});
	this.displayURL(
		this.settings.contentURL + page.id + ".html", 
		pageContainer,
		function (data) {
			pageMarkup = data;
			initPage();					
	});
	
};

DKI.PresentationPlayer.prototype.loadQuestion = function (position, question, callback) {
	var self = this;
	var callbackFunction = function () {
		this.contentPage[position] = new DKI.TestQuestion(this, this.contentPage[position], question);
		if (callback) {
			callback.call(self);
		}
	};
	this.loadPage(position, question.page, callbackFunction);
};

/**
 * Load the current question into the content frame
 * @method loadCurrentQuestion
 */
DKI.PresentationPlayer.prototype.loadCurrentQuestion = function () {
	var currentQuestion = this.dataStorage.getCurrentQuestion();
	var nextQuestion = this.dataStorage.getNextQuestion();
	var previousQuestion = this.dataStorage.getPreviousQuestion();
	this.applyTransitionClass("next", this.transitions.jump);
	this.loadQuestion("next", currentQuestion, function () {
		this.shuffleForward();
		if (this.inReview) {
			this.questionPage.showReview(
				this.dataStorage.assessment.prepost,
				this.dataStorage.getCurrentQuestion().userChoices,
				this.dataStorage.getCurrentQuestion().options,
				this.dataStorage.getCurrentQuestion());
		}
		this.applyTransitionClass("current", this.transitions.page);
		if (nextQuestion) {
			this.applyTransitionClass("next", this.transitions.page);
			this.loadQuestion("next", nextQuestion);
		}
		else {
			this.loadModuleEnd();
		}
		if (this.behaviour.transcripts) {
			if (currentQuestion.page.transcript) {
				this.navigation.enableTranscriptButton();
			}
			else {
				this.navigation.disableTranscriptButton();
			}
		}
		this.menu.selectQuestion(currentQuestion);
		this.assessmentState = "next";
		this.questionFeedbackClosed = "false";
		this.initBarsAssessment();
		$(this).trigger(this.events.questionPageLoaded);
		$(this).trigger(this.events.pageLoaded);
	});
};

/**
 * Load the module end screen into the content frame
 * @method loadModuleEnd
 * @param lastModule {Object} The module that just ended
 */
DKI.PresentationPlayer.prototype.loadModuleEnd = function () {
	var that = this;
	this.applyTransitionClass("next", this.transitions.end);
	this.displayURL(this.settings.endModURL, this.pageFrames.next, function () {
		that.contentPage.next = DKI.EndModule;
		DKI.EndModule.init(that);
	});
	this.forward = this.startModuleEnd;
};

DKI.PresentationPlayer.prototype.loadPageActions = function (id, callback) {
	var url = this.settings.actionsURL + id + this.settings.actionsPrefix;
	var callBackObj = {
		success: function () {},
		error: function () {},
	
		complete: function () {}
	};
	if (callback) {
		if (typeof(callback) == "function") {
			callBackObj.success = callback;	
		}
		else {
			if (callback.success) {
				callBackObj.success = callback.success;
			}
			if (callback.error) {
				callBackObj.error = callback.error;
			}
			if (callback.complete) {
				callBackObj.complete = callback.complete;
			}
		}
	};
	$.get(url, function (data) {
	}, "json").success(callBackObj.success).error(callBackObj.error).complete(callBackObj.complete);
};

/**
 * Resumes learning content, called from endscreens
 * @method resume
 */
DKI.PresentationPlayer.prototype.resume = function (fromPre) {
	var self = this;
	var resumeTestOnly = function () {
		var status = self.dataStorage.lessonStatus;
		if (status === "passed" || status === "failed" || self.dataStorage.allTestsAttempted()) {
			self.startCourseEnd();
		}
		else if (self.dataStorage.goNextModule()){
			self.launchTest(self.dataStorage.currentModule.id, "post");
		}
		else {
			self.dataStorage.onCourseComplete();
			self.startCourseEnd();
		}
	};
	var resumeContent = function (fromPreTest) {
		var tempFrame = self.pageFrames.current;
		self.inAssessment = false;
		self.inEndScreen = false;
		if (self.loadState.next === "loading") {
			if (!self.queuedAction) {
				self.queuedAction = "resume";
			}
			return true;
		}
		if (self.reachedEnd) {
			self.startCourseEnd();
		}
		else {
			//When we come back from a pretest, we're going to display the current page.
			if (!fromPreTest) {
				self.dataStorage.goNextPage();
			}
			if (self.checkPreTest()) {
				self.launchTest(self.dataStorage.currentModule.id, "pre");
			}
			else {
				self.contentFrame.removeClass("back").addClass("forward");
				self.applyTransitionClass("next", self.transitions.page);
				self.contentPage.current = self.contentPage.next;
				self.pageFrames.current = self.pageFrames.next;
				self.pageFrames.next = tempFrame;
				$(self.pageFrames.current).removeClass("next").addClass("current");
				$(self.pageFrames.next).removeClass("current").addClass("next");
				self.contentPage.current.start();
				self.dataStorage.setCurrentPageComplete();
				self.initBars();
				self.navigation.setCurrentScreen(self.dataStorage.currentModulePage);
			
				// added ticket 22864: always ensure buttons are enabled as we can jump from test to page via the menu
				$(self).trigger(self.events.contentPageLoaded);
				$(self).trigger(self.events.pageLoaded);	
				if (self.dataStorage.previousPage) {
					self.loadPage("previous", self.dataStorage.previousPage);
				}
				if (self.dataStorage.isEndOfCourse()) {
					self.loadCourseEnd();
				}
				else if (self.dataStorage.isEndOfModule()) {
					self.loadModuleEnd();
				}
				else {
					self.loadPage("next", self.dataStorage.nextPage);
				}
			}
		}
	};

	if (this.course.testOnly == "1") {
		resumeTestOnly();
	}
	else {
		resumeContent(fromPre);
	}
};

/**
 * Called when player is launched to resume from the last bookmark, or the first page of the course.
 * This function is responsible for actually loading the content page when the player is initialized.
 */
DKI.PresentationPlayer.prototype.resumeBookmark = function () {
	var bookmark = this.dataStorage.bookmark;
	var status = this.dataStorage.lessonStatus;
	if (this.course.testOnly == "1") {
		if (status === "passed" || status === "failed" || this.dataStorage.allTestsAttempted()) {
			this.resumeToCourseEnd();
		}
		else if (bookmark && bookmark != "" && this.dataStorage.jumpToModule(bookmark)) {
			this.launchTest(bookmark, "post");
		}
		else {
			this.launchTest(this.course.modules[0].id, "post");
		}
	}
	else {
		this.dataStorage.resume(this.behaviour.autoResume);
		this.setModuleDisplay();
		this.loadCurrentPage();
	}
};

DKI.PresentationPlayer.prototype.resumeToCourseEnd = function () {
	self = this;
	this.contentPage.current = DKI.EndCourse;
	DKI.EndCourse.init(this, this.pageFrames.current);
	this.displayURL(this.settings.endCourseURL, this.pageFrames.current, function () {
		self.initBarsEndScreen();
		self.hideTree();
		DKI.EndCourse.start();
	});
};

/**
 * Shuffles the content frames and page objects forward, moving the previous frame into the next postion
 */
DKI.PresentationPlayer.prototype.shuffleForward = function () {
	var tempFrame = this.pageFrames.previous;
	//Add the 'forward' class, so we don't see the previous frame transition to the next position
	this.contentFrame.removeClass("back").addClass("forward");
	this.pageFrames.previous = this.pageFrames.current;
	this.pageFrames.current = this.pageFrames.next;
	this.pageFrames.next = tempFrame;
	this.contentPage.previous = this.contentPage.current;
	this.contentPage.current = this.contentPage.next;
	this.contentPage.next = null;
	tempFrame.innerHTML = "";
	$(this.pageFrames.previous).removeClass("current").addClass("previous");
	$(this.pageFrames.current).removeClass("next").addClass("current");
	$(this.pageFrames.next).removeClass("previous").addClass("next");
	this.contentFrame.height(this.course.height);
	$(this.pageFrames.previous).height(this.course.height);
	if ($(document.body).hasClass('mobile')) {		
		$("#headerContainer").css("position", "relative");
		$(document.body).scrollTop(0);
		if (Modernizr["position-fixed"]) {
			$("#headerContainer").css("position", "fixed");
		}
	}	
	this.contentPage.current.start();
	if (this.contentPage.previous !== null && this.contentPage.previous.reset) {
		this.contentPage.previous.reset();
	}
};

DKI.PresentationPlayer.prototype.shuffleBack = function () {
	var tempFrame = this.pageFrames.next;
	//Add the 'back' class, so we don't see the next frame transition to the previous position
	this.contentFrame.removeClass("forward").addClass("back");
	this.pageFrames.next = this.pageFrames.current;
	this.pageFrames.current = this.pageFrames.previous;
	this.pageFrames.previous = tempFrame;
	this.contentPage.next = this.contentPage.current;
	this.contentPage.current = this.contentPage.previous;
	this.contentPage.previous = null;
	tempFrame.innerHTML = "";
	$(this.pageFrames.next).removeClass("current").addClass("next");
	$(this.pageFrames.current).removeClass("previous").addClass("current");
	$(this.pageFrames.previous).removeClass("next").addClass("previous");
	this.contentFrame.height(this.course.height);
	$(this.pageFrames.next).height(this.course.height);
	if ($(document.body).hasClass('mobile')) {			
		$("#headerContainer").css("position", "relative");
		$(document.body).scrollTop(0);
		if (Modernizr["position-fixed"]) {
			$("#headerContainer").css("position", "fixed");
		}
	}
	this.contentPage.current.start();
	if (this.contentPage.next.reset) {
		this.contentPage.next.reset();
	}

};

/**
 * Called to start the module end screen, setup the forward function and load the next page
 */
DKI.PresentationPlayer.prototype.startModuleEnd = function () {
	var forAssessment = this.inAssessment || this.inReview;
	this.inReview = false;
	this.inAssessment = false;
	$(".questionScoreDisplay").hide();
	$(".submitButton").hide();	
	if (forAssessment) {
		if ((this.course.testOnly == "1" && this.dataStorage.allTestsAttempted()) || 
				(this.course.testOnly != "1" && !this.dataStorage.nextPage && this.dataStorage.assessment.prepost !== "pre")) {
			this.reachedEnd = true;
		}
	}
	this.setModuleDisplay();
	this.clearObjectDisplay();
	this.initBarsEndScreen();
	this.hideTree();
	this.forward = this.contentForward;
	//Shuffle frames
	this.shuffleForward();
	if (this.reachedEnd) {
		this.loadCourseEnd();
	}
	else {
		this.applyTransitionClass("next", this.transitions.end);
		if (this.course.testOnly != "1") {
			if (forAssessment && this.dataStorage.assessment.prepost === "pre") {
				this.loadPage("next", this.dataStorage.currentPage);
			}
			else {
				this.loadPage("next", this.dataStorage.nextPage);
			}
			this.forward = this.contentForward;
			this.back = this.contentBack;
		}
	}
};

/**
 * Shuffles the course end screen to the current slot.
 */
DKI.PresentationPlayer.prototype.startCourseEnd = function () {
	this.initBarsEndScreen();
	//We want menu enabled for course endscreen
	this.navigation.showMenuButton();
	this.hideTree();
	this.shuffleForward();
	this.pageFrames.next.innerHTML = "";	
};

DKI.PresentationPlayer.prototype.setPlayerSize = function () {	
	var totalHeight = $("#headerContainer").height() + $("#footerContainer").height() + parseInt(this.course.height, 10);
	var totalWidth = this.course.width;
	var deltaX = totalWidth - $(window).width();
	var deltaY = totalHeight - $(window).height();
	this.menu.setHeight(parseInt(this.course.height, 10) - 20);
	if (this.behaviour.resizeWindow) {
		//Case 22790 - Checking to see screen real-estate.. if were in a crap resolution, just resize it to the maximum the window can take.
		if(screen.availHeight <= 800){
			top.window.resizeTo(totalWidth, screen.availHeight);
		}
		else{
			top.window.resizeBy(deltaX, deltaY);
		}
		if (window.scrollHeight > window.clientHeight) {
			alert("Too small");
			$("body").overflow("auto");
		}
	}
};

/* Event handlers */

DKI.PresentationPlayer.prototype.onEndReady = function (e) {
	var currentPage = this.dataStorage.currentPage;

	if (currentPage && !this.inAssessment && !currentPage.disableNext) {
		this.navigation.showForwardButton();
	}
};

DKI.PresentationPlayer.prototype.onFeedbackSubmit = function (text, callback) {
	
	callback = callback==undefined? function(){} : callback;

	var	page = this.inAssessment ? this.dataStorage.assessment.currentQuestion.page : this.dataStorage.currentPage;

	var ajaxArgs = {
		type:"PUT",
		headers: {
			"Content-Type": "application/json; charset=UTF-8"
		},
		processData: false,
		url: "../api/student.cfm/page/" + page.id + "/note",
		dataType: "json",
		success: function(data, textStatus, jqXHR) {
			callback(data)
		},
		context: this
	};
	if (/\S/.test(text)) {
		ajaxArgs.data = JSON.stringify({description: text});
		$.ajax(ajaxArgs);
	}

	return false;
}

/**
 * Handler for a question's load event
 * @method onQuestionLoaded
 * @param url {string} The URL of the question to display
 */
DKI.PresentationPlayer.prototype.onQuestionLoaded = function (url) {
	this.navigation.setCurrentScreen(this.currentTest.currentQuestionDisplay);
	this.displayURL(url, this.currentFrame);
	return true;
};

DKI.PresentationPlayer.prototype.onQuestionFeedbackClosed = function () {
	if (this.assessmentState === "next") {
		this.loadCurrentQuestion();
	}
	else if (this.assessmentState === "complete") {
		this.loadModuleEnd();
	}
	else {
		this.questionFeedbackClosed = true;
	}
};

DKI.PresentationPlayer.prototype.onPageClicked = function (pageid) {

	if (this.canJump(pageid)) {
		this.hideTree();

		//Need to verify then complete jump to page first.
		this.jumpTo(pageid);
	}
	else {
		alert(this.strings.noNavLabel);
	}
};

DKI.PresentationPlayer.prototype.onPageReady = function (e, page) {
	if (this.inAssessment || this.inReview) {
		this.onQuestionPageReady(page);
	}
	else {
		this.onContentPageReady(page);
	}
};

DKI.PresentationPlayer.prototype.onContentPageReady = function (page) {
	var nextPageID = this.dataStorage.nextPage ? parseInt(this.dataStorage.nextPage.id, "10") : null;
	var prevPageID = this.dataStorage.previousPage ? parseInt(this.dataStorage.previousPage.id, "10") : null;
	var pageID = parseInt(page.id);
	var currentPage = this.dataStorage.currentPage;

	if (pageID === nextPageID && !currentPage.disableNext && !this.inEndScreen) {
		this.navigation.showForwardButton();
	}
	else if (pageID === prevPageID && !currentPage.disableBack) {
		this.navigation.showBackButton();
	}
};

DKI.PresentationPlayer.prototype.onQuestionPageReady = function (page) {
	//var nextQuestionID = this.dataStorage.getNextQuestion() ? this.dataStorage.getNextQuestion().page.id : null;
	
	if (this.inReview) {
		this.navigation.showForwardButton();
	}
};

/* Navigation methods */

/**
 * Moves learning content back by one screen
 * @method contentBack
 */
DKI.PresentationPlayer.prototype.contentBack = function () {
	var page;
	//Whenever we go backwards, we have to end up at least one page from the end.  Ensure this is false.
	this.reachedEnd = false;
	this.contentAPI.hideBoth();
	if (this.loadState.previous === "loading") {
		if (!this.queuedAction) {
			this.queuedAction = "back";
		}
		return true;
	}
	this.dataStorage.goPreviousPage();
	page = this.dataStorage.currentPage;
	this.shuffleBack();
	this.initBars();
	this.dataStorage.setCurrentPageComplete();
	this.navigation.setCurrentScreen(this.dataStorage.currentModulePage);
	//Ensure the 'forward' method is contentForward.  If an endscreen was preloaded before 'previous' action
	//was executed, it may be pointing to a 'start endscreen' action.
	this.forward = this.contentForward;
	// added ticket 22864: always ensure buttons are enabled as we can jump from test to page via the menu
	$(this).trigger(this.events.contentPageLoaded);
	$(this).trigger(this.events.pageLoaded);	
	//Determine if an endscreen or page should be loaded in the 'next' slot
	if (this.dataStorage.isEndOfCourse()) {
		this.loadCourseEnd();
	}
	else if (!this.behaviour.skipEnd && this.dataStorage.isEndOfModule()) {
		this.loadModuleEnd();
	}

	if (this.dataStorage.previousPage) {
		this.applyTransitionClass("previous", this.transitions.page);
		this.loadPage("previous", this.dataStorage.previousPage);
	}
	return true;
};

/**
 * Advances learning content one screen
 * @method contentForward
 */
DKI.PresentationPlayer.prototype.contentForward = function() {
	var page;
	var currentModule = this.dataStorage.currentModule;
	var transition = this.transitions.page + " next";
	this.contentAPI.hideBoth();
	if (this.loadState.next === "loading") {
		if (!this.queuedAction) {
			this.queuedAction = "forward";
		}
		return true;
	}
	this.dataStorage.goNextPage();
	if (currentModule !== this.dataStorage.currentModule && this.checkPreTest(this.dataStorage.currentModule)) {
		this.launchTest(this.dataStorage.currentModule.id, "pre");
	}
	else {
		this.applyTransitionClass(transition);
		this.shuffleForward();
		this.dataStorage.setCurrentPageComplete();
		this.navigation.setCurrentScreen(this.dataStorage.currentModulePage);
		
		// added ticket 22864: always ensure buttons are enabled as we can jump from test to page via the menu
		$(this).trigger(this.events.contentPageLoaded);
		$(this).trigger(this.events.pageLoaded);	
		//Determine if an endscreen or page should be loaded in the 'next' slot
		if (this.dataStorage.isEndOfCourse()) {
			this.loadCourseEnd();
		}
		else if (!this.behaviour.skipEnd && this.dataStorage.isEndOfModule()) {
			this.loadModuleEnd();
		}
		else if (this.dataStorage.nextPage) {
			this.applyTransitionClass("next", this.transitions.page);
			this.loadPage("next", this.dataStorage.nextPage);
		}
		this.initBars();
	}

	return true;
};

/**
 * Reloads the current content page
 */
DKI.PresentationPlayer.prototype.contentReload = function () {
	this.contentPage.current.reset();
	this.contentPage.current.start();
};

/**
 * Moves the current assessment to the next question
 */
DKI.PresentationPlayer.prototype.assessmentForward = function () {
	var nextQuestion;
	var transition = this.transitions.page + " next";
	this.dataStorage.goNextQuestion();
	this.applyTransitionClass(transition);
	this.contentAPI.hideBoth();
	if (this.loadState.next === "loading") {
		if (!this.queuedAction) {
			this.queuedAction = "forward";
		}
		return true;
	}
	this.shuffleForward();
	if (this.inReview) {
		this.questionPage.showReview(
			this.dataStorage.assessment.prepost,
			this.dataStorage.getCurrentQuestion().userChoices,
			this.dataStorage.getCurrentQuestion().options,
			this.dataStorage.getCurrentQuestion());
	}
	this.initBarsAssessment();		
	nextQuestion = this.dataStorage.getNextQuestion();
	if (nextQuestion) {
		this.applyTransitionClass("next", this.transitions.page);
		this.loadQuestion("next", nextQuestion); 
	}
	else {
		this.loadModuleEnd();
	}
	$(this).trigger(this.events.pageLoaded);
	$(this).trigger(this.events.questionPageLoaded);
	return true;
};

/**
 * Moves the current assessment to the previous question
 */
DKI.PresentationPlayer.prototype.assessmentBack = function () {
	var prevQuestion;
	var transition = this.transitions.page + " previous";
	this.applyTransitionClass(transition);
	this.contentAPI.hideBoth();
	if (this.loadState.previous === "loading") {
		if (!this.queuedAction) {
			this.queuedAction = "back";
		}
		return true;
	}
	this.dataStorage.goPreviousQuestion();
	this.shuffleBack();
	prevQuestion = this.dataStorage.getPreviousQuestion();
	if (prevQuestion) {
		this.applyTransitionClass("previous", this.transitions.previous);
		this.loadQuestion("previous", prevQuestion); 
		this.contentAPI.showBoth();
	};
	$(this).trigger(this.events.pageLoaded);
	$(this).trigger(this.events.questionPageLoaded);
	return true;
};

/**
 * Hide the course menu
 * @method hideTree
 */
DKI.PresentationPlayer.prototype.hideTree = function () {
	this.menu.tabSet.hide()
	this.navigation.setMenuState(false);
};

/**
 * Displays the course menu
 * @method showTree
 */
DKI.PresentationPlayer.prototype.showTree = function () {
	this.menu.tabSet.show()
	this.menu.scrollToSelected();
	this.navigation.setMenuState(true);
};

/**
 * Toggles the current state of course audio from on to off
 */
DKI.PresentationPlayer.prototype.toggleAudio = function (audio) {
	if (audio === "on") {
		this.audio = true;
	}
	else {
		this.audio = false;
	}
	if (this.contentPage.current.setAudio) {
		this.contentPage.current.setAudio(this.audio);
	}
};

/**
 * Displays the page audio transcript
 */
DKI.PresentationPlayer.prototype.showTranscript = function () {
	this.menu.selectedTab("transcriptTab");
	this.showTree();
};

/**
 * Hides the page audio transcript
 */
DKI.PresentationPlayer.prototype.hideTranscript = function () {	
	this.hideTree();
};

/**
 * Toggles the shown/hidden state of the page audio transcript
 */
DKI.PresentationPlayer.prototype.toggleTranscript = function () {
	if (this.menu.selectedTab() === "transcriptTab" && !this.menu.tabSet.isHidden()){
		this.hideTranscript();
	}
	else{
		this.showTranscript();
	}
};

/**
 * Hide the course outline
 * @method hideOutline
 */
DKI.PresentationPlayer.prototype.hideOutline = function () {
	this.hideTree();
};

/**
 * Displays the course outline
 * @method showOutline
 */
DKI.PresentationPlayer.prototype.showOutline = function () {
	this.menu.selectedTab("outlineTab");
	this.showTree();
};

/**
 * Toggles the shown/hidden state of the outline
 */
DKI.PresentationPlayer.prototype.toggleOutline = function () {
	if (this.menu.selectedTab() === "outlineTab" && !this.menu.tabSet.isHidden()){
		this.hideOutline();
	}
	else{
		this.showOutline();
	}
};

/** 	
 * jumps to the subeo passed in and sets completion on everything before
 * @param [string] subeoId the subeoid to jump to
 */
DKI.PresentationPlayer.prototype.jumpToSubeo = function (subeoId) {
	var page = this.dataStorage.getPageFromSubeo(subeoId);
	this.initBars();
	this.hideTree();
	this.dataStorage.completePreviousPages(page);
	this.jumpTo(page.id);
};

/* Navigation helper methods */

DKI.PresentationPlayer.prototype.setCurrentObjectComplete = function () {
	this.dataStorage.setCurrentObjectComplete();
};

/**
 * Displays the current module details in the navigation bars
 * @method setModuleDisplay
 * @private
 */
DKI.PresentationPlayer.prototype.setModuleDisplay = function () {
	this.navigation.setModuleName(this.dataStorage.currentModule.name);
	this.navigation.setTotalScreens(this.dataStorage.currentModulePageTotal);
};

/**
 * Displays the current learning object details in the navigation bars
 * @method setObjectDisplay
 * @private
 */
DKI.PresentationPlayer.prototype.setObjectDisplay = function () {
	text = this.dataStorage.currentObject ? this.dataStorage.currentObject.name : "";
	this.navigation.setObjectName(text);
};

DKI.PresentationPlayer.prototype.clearObjectDisplay = function () {
	this.navigation.setObjectName("");
};

/* 'Private' methods.  Helpers that are publically available, but only intended to be call from within instance methods */

DKI.PresentationPlayer.prototype.canJump = function (pageID) {
	var nextPage;
	if (!this.behaviour.forceSequential) {
		return true;
	}
	else {
		nextPage = this.dataStorage.getPrecedingPage(pageID);
		if (!nextPage || nextPage.complete) {
			return true;
		}
		else {
			return false;
		}
	}
};

DKI.PresentationPlayer.prototype.applyCssEffects = {
	allowed :  function(){
		return (($.browser.msie && Number($.browser.version.substr(0,1))>6) || !$.browser.msie);
	}(),
	init : function(){
		if(this.allowed){
			reflections=$('.cssFX-slctr-Reflect');
			for(var r=0;r < reflections.length; r++){
				var images=$($('.cssFX-slctr-Reflect')[r]).find('img');
				if(images.length==2 && $(reflections[r]).data('rolloverassetid')!=null){
					//alert($(images[1]).attr('id') )
					var id = '#'+$(images[1]).attr('id') 
					$(id).show();
					$(id).reflect({
						height:1/3,opacity:0.5,
						attr:{
							cls:'dki-image-rollover-reflected',
							style:'filter:inherit;left: 0px; top: 0px; width: inherit !important; height: inherit !important; right:0px;bottom:0px;overflow: hidden; position: absolute;display:none;z-index:2'
						}
					})
					$(id).attr('class','reflected') //dki-image-rollover '
				};
				
				$(images[0]).reflect({
					height:1/3,opacity:0.5,
					attr:{
						cls:'dki-image-root-reflected',
						style:'filter:inherit;left: 0px; top: 0px; width: inherit !important; height: inherit !important; right:0px;bottom:0px; overflow: hidden; z-index: 1; position: absolute;'
					}
				})	
			}
		}else{
			var reflections=$('.cssFX-slctr-Reflect');
			for(var r=0;r < reflections.length; r++){
				$(reflections[r]).removeClass('cssFX-slctr-Reflect');
				$(reflections[r]).removeClass('cssFX-Reflect')
				$(reflections[r]).removeClass('cssFX-MirrorRounded')
				$(reflections[r]).height(
					Math.round($(reflections[r]).height()*.7)
				)
			}			
		}
	}
};

DKI.PresentationPlayer.prototype.openModal = function(url) {
	if ($("body").hasClass("phone")) {
		var newWin = window.open(url, '_blank');
	}
	else{
		$("#modalFrame").attr("src", url);
		$("#modalDisabler").css({
			"top": $("#headerContainer").height() + "px",
			"height": $("#contentFrame").height() + "px"
		});
		$("#modalDisabler").fadeIn();
	}
};

// Static values
DKI.PresentationPlayer.transitionClasses = "none slide fade flip pop";
